@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 import gtk.glade, gtk, gobject, os.path, time, os, sys, re, threading, gtk.gdk, Image, StringIO, pango, string, keyEditor, traceback
 import recipeManager
+import nutrition.nutrition, nutrition.nutritionGrabberGui
 import exporters.printer as printer
 import prefs, prefsGui, shopgui, reccard, convertGui, fnmatch, tempfile
 import exporters, importers
-import convert, WidgetSaver, version
+import convert, WidgetSaver, version, ratingWidget
 import importers.mastercook_importer as mastercook_importer
 import dialog_extras as de
 import treeview_extras as te
@@ -14,6 +15,7 @@ from gglobals import *
 from recindex import RecIndex
 import exporters.recipe_emailer as recipe_emailer
 import locale, gettext
+from timer import show_timer
 _ = gettext.gettext
 from defaults import lang as defaults
 
@@ -30,7 +32,6 @@ else:
 
 import gettext_setup
 
-
 try:
     import rtf_exporter
     rtf=True
@@ -38,19 +39,51 @@ except ImportError:
     debug('No RTF support',0)
     rtf=False
 
+def check_for_data_to_import (rm):
+    if rm.fetch_len(rm.rview)==0:
+        try:
+            import legacy_db
+        except ImportError:
+            print "Not trying to update."
+            print "We had an import error."
+            import traceback; traceback.print_exc()
+        else:
+            pd = de.ProgressDialog(label=_('Importing old recipe data'),
+                                   sublabel=_('Importing recipe data from a previous version of Gourmet into new database.'),
+                                   )
+            def set_prog (p,msg=None):
+                p=float(p)
+                pd.set_progress(p,msg)
+                while gtk.events_pending():
+                    gtk.main_iteration()
+            legacy_db.backup_legacy_data(gourmetdir, pd, set_prog)
+            backup_file = os.path.join(gourmetdir,'GOURMET_DATA_DUMP')
+            if os.path.exists(backup_file):
+                import upgradeHandler        
+                pd.show()
+                upgradeHandler.import_backup_file(
+                    rm,backup_file,set_prog
+                    )
+                os.rename(backup_file,backup_file+'.ALREADY_LOADED')
+                pd.hide()
+                pd.destroy()
+        
 class RecGui (RecIndex):
     """This is the main application. We subclass RecIndex, which handles displaying a list of
     recipes and searching them (a functionality we need in a few other places, such as when
     calling a recipe as an ingredient or when looking through the "trash". """
+
     def __init__ (self,splash_label=None):
         # used on imports to make filtering wait until
         # we are all done.
         self.wait_to_filter=False
         try:
             import gnome
+            # The following allows accessibility support to work for
+            # some unknown reason
             # apparently some outdated GNOME bindings are
             # missing this.
-            if hasattr(gnome,'program_init'): 
+            if hasattr(gnome,'program_init'):
                 gnome.program_init(version.appname,version.version)
             else:
                 debug(
@@ -68,6 +101,8 @@ class RecGui (RecIndex):
             self.prefs,
             buttons={'clear_remembered_optional_button':lambda *args: self.forget_remembered_optional_ingredients()}
             )
+        self.prefsGui.apply_prefs_dic['recipes_per_page']= lambda p,v: getattr(getattr(self,'rmodel'),
+                                                                               'change_items_per_page')(v)
         self.update_splash(_("Loading graphical interface..."))        
         gtk.glade.bindtextdomain('gourmet',DIR)
         gtk.glade.textdomain('gourmet')
@@ -106,16 +141,16 @@ class RecGui (RecIndex):
         for a,l,w in REC_ATTRS:
             self.rtcolsdic[a]=l
             self.rtwidgdic[a]=w
-        self.rtcols=map(lambda r: r[0], REC_ATTRS)
+        self.rtcols = [r[0] for r in REC_ATTRS]
         self.update_splash(_("Loading recipe database..."))
         self.init_recipes()
         self.update_splash(_("Setting up recipe index..."))
         RecIndex.__init__(self,
-                          model=self.rmodel,
+                          #model=self.rmodel,
                           glade=self.glade,
                           rd=self.rd,
                           rg=self,
-                          editable=True)
+                          editable=False)
 
         # we need an "ID" to add/remove messages to/from the status bar
         self.pauseid = self.stat.get_context_id('pause')
@@ -130,7 +165,8 @@ class RecGui (RecIndex):
             'showList' : lambda *args: self.app.present(),            
             'new' : self.new,
             'defaultsave': self.save_default,
-            'export' : self.exportg,
+            'export' : self.export_selected_recs,
+            'export_all': self.export_all_recs,
             'import' : self.importg,
             'import_webpage': self.import_webpageg,
             'quit' : self.quit,
@@ -146,7 +182,8 @@ class RecGui (RecIndex):
             'email':self.email_recs,
             #'email_prefs':self.email_prefs,
             'showDeletedRecipes':self.show_deleted_recs,
-            'emptyTrash':self.empty_trash
+            'emptyTrash':self.empty_trash,
+            'on_timer': lambda *args: show_timer(),
 #            'shopCatEditor': self.showShopEditor,            
             })
         # self.rc will be a list of open recipe cards.
@@ -157,9 +194,16 @@ class RecGui (RecIndex):
         ## make sure the focus is where it ought to be...
         self.app.present()
         self.srchentry.grab_focus()
+        self.srchentry.connect('activate',self.search_entry_activate_cb)
         self.update_splash(_("Done!"))
         self.threads = 0
-        
+
+    def search_entry_activate_cb (self, *args):
+        if self.rmodel._get_length_()==1:
+            self.recTreeSelectRec()
+        else:
+            self.limit_search()
+            
     def update_splash (self, text):
         """Update splash screen on startup."""
         debug("Setting splash text: %s"%text,3)
@@ -169,7 +213,7 @@ class RecGui (RecIndex):
             gtk.main_iteration()
         
     def del_rc (self, id):
-        """Delete recipe with ID=id"""
+        """Forget about recipe card identified by id"""
         if self.rc.has_key(id):
             del self.rc[id]
         self.updateViewMenu()
@@ -272,13 +316,12 @@ class RecGui (RecIndex):
                 sublabel += _('\nWebsite: %s')%website
             import xml.sax.saxutils
             de.show_message(label=xml.sax.saxutils.escape('%s %s'%(appname,myversion)),
-                            #sublabel=xml.sax.saxutils.escape(sublabel)) #old line that messed with our about dialog
                             sublabel=sublabel) #new line that leaves strings as they are.
             
     def show_help (self, *args):
         de.show_faq(HELP_FILE)
 
-    def show_progress_dialog (self, thread, prog_dialog_kwargs={},message=_("Import paused"),
+    def show_progress_dialog (self, thread, progress_dialog_kwargs={},message=_("Import paused"),
                            stop_message=_("Stop import")):
         """Show a progress dialog"""
         if hasattr(thread,'name'): name=thread.name
@@ -289,31 +332,29 @@ class RecGui (RecIndex):
                     ('pause',self.pause_cb),
                     ('stop',self.stop_cb),
                     ('modal',False),]:
-            if not prog_dialog_kwargs.has_key(k):
-                prog_dialog_kwargs[k]=v
-        if not hasattr(self,'prog_dialog') or not self.prog_dialog:
-            self.prog_dialog = de.progressDialog(**prog_dialog_kwargs)
-            self.prog = self.prog_dialog.progress_bar
+            if not progress_dialog_kwargs.has_key(k):
+                progress_dialog_kwargs[k]=v
+        if not hasattr(self,'progress_dialog') or not self.progress_dialog:
+            self.progress_dialog = de.ProgressDialog(**progress_dialog_kwargs)
+            self.prog = self.progress_dialog.progress_bar
         else:
-            self.prog_dialog.reassign_buttons(pausecb=prog_dialog_kwargs['pause'],
-                                              stopcb=prog_dialog_kwargs['stop'])
-            
-            self.prog_dialog.reset_label(prog_dialog_kwargs['label'])
+            self.progress_dialog.reassign_buttons(pausecb=progress_dialog_kwargs['pause'],
+                                              stopcb=progress_dialog_kwargs['stop'])
+            self.progress_dialog.reset_label(progress_dialog_kwargs['label'])
         self.pause_message = message
         self.stop_message = stop_message
         self.thread = thread        
-        self.prog_dialog.show()
-        self.prog_dialog.connect('close',lambda *args: setattr(self,prog_dialog,None))
+        self.progress_dialog.show()
+        self.progress_dialog.connect('close',lambda *args: setattr(self,progress_dialog,None))
         #self.pauseButton.show()
         #self.stopButton.show()
         
     def hide_progress_dialog (self):
         """Make the progress dialog go away."""
-        print 'Called hide_progress_dialog!'
-        if hasattr(self,'prog_dialog') and self.prog_dialog:
-            self.prog_dialog.hide()
-            self.prog_dialog.destroy()
-            self.prog_dialog = None
+        if hasattr(self,'progress_dialog') and self.progress_dialog:
+            self.progress_dialog.hide()
+            self.progress_dialog.destroy()
+            self.progress_dialog = None
             
     def quit (self, *args):
         """Close down shop, giving user option of saving changes and
@@ -354,7 +395,9 @@ class RecGui (RecIndex):
                             return True
                 if not use_threads:
                     for t in self._threads:
-                        try: t.terminate()
+                        try:
+                            t.terminate()
+                            self.threads = self.threads - 1
                         except: return True
             else:
                 return True
@@ -379,10 +422,10 @@ class RecGui (RecIndex):
         (or possibly make sense of it themselves!)."""
         try:
             self.rd = recipeManager.RecipeManager(**recipeManager.dbargs)
-            # if we are using metakit, initiate autosave stuff
-            if self.rd.db=='metakit':
-                # autosave every 3 minutes (milliseconds * 1000 milliseconds/second * 60 seconds/minute)
-                gobject.idle_add(self.rd.save,1000*60*3)
+            check_for_data_to_import(self.rd)
+            # initiate autosave stuff
+            # autosave every 3 minutes (milliseconds * 1000 milliseconds/second * 60 seconds/minute)
+            gobject.timeout_add(1000*60*3,lambda *args: self.rd.save() or True)
         except:
             self.prefs['db_backend'] = None
             self.prefs.save()
@@ -399,18 +442,24 @@ class RecGui (RecIndex):
         # connect hooks to modify our view whenever and
         # whenceever our recipes are updated...
         self.rd.modify_hooks.append(self.update_rec_iter)
-        self.rd.add_hooks.append(self.new_rec_iter)
+        #self.rd.add_hooks.append(self.new_rec_iter)
         self.rd.delete_hooks.append(self.delete_rec_iter)
         # a flag to make deleting multiple recs
         # more efficient...
         self.doing_multiple_deletions=False
         #self.conv = rmetakit.mkConverter(self.rd)
         self.conv = convert.converter()
+        # initialize our nutritional database
+        nutrition.nutritionGrabberGui.check_for_db(self.rd)
+        self.nd = nutrition.nutrition.NutritionData(self.rd,self.conv)
+        self.rd.nd = self.nd
+        # initialize star-generator for use elsewhere
+        self.star_generator = ratingWidget.StarGenerator()
         # we'll need to hand these to various other places
         # that want a list of units.
         self.umodel = convertGui.UnitModel(self.conv)
         self.inginfo = reccard.IngInfo(self.rd)
-        self.create_rmodel(self.rd.rview)
+        #self.create_rmodel(self.rd.rview)
         self.sl = shopgui.ShopGui(self, conv=self.conv)
         self.sl.hide()
 
@@ -423,97 +472,29 @@ class RecGui (RecIndex):
 
     def rectree_click_cb (self, tv, event):
         """Display popup button for right-click on rectree."""
-        debug("rectree_click_cb (self, tv, event):",5)
+        debug("rectree_click_cb (self, event):",5)
         if event.button==3:
-            self.popup_rmenu()
+            self.popup_rmenu(event)
             return True
-        if event.button==1 and event.type ==gtk.gdk._2BUTTON_PRESS:
-            self.recTreeSelectRec()
+        #if event.button==1 and event.type ==gtk.gdk._2BUTTON_PRESS:
+        #    self.recTreeSelectRec()
 
     def reset_rtree (self):
         """Re-create our recipe model."""
         debug("reset_rtree (self):",5)
-        #self.rmodel=self.create_rmodel(self.rd.rview)
-        #self.rectree.set_model(self.rmodel)
-        self.update_rmodel(self.rd.rview)
-        self.search()
-        self.selection_changed()
+        self.setup_search_views()
+        self.rmodel.change_view(self.lsrchvw)
+        self.rectree.set_model(self.rmodel)
         self.set_reccount()
-        #for r in rview:
-        #    if not r.id in ids:
-        #        # if we're already there...
-        #        iter=self.rmodel.append(None)
-        #        self.set_iter_from_rec(r,iter)
 
     def new_rec_iter (self, rec):
-        debug("new_rec_iter called",5)
-        iter = self.rmodel.append(None)
-        self.set_iter_from_rec(rec,iter)
+        self.reset_rtree()
 
     def delete_rec_iter (self, rec):
         if self.doing_multiple_deletions: return
-        else: self.rmodel_filter.refilter()
-        #for row in self.rmodel:
-        #    if row[0].id==rec.id:
-        #        debug("delete_rec_iter found the row to delete",5)
-        #        dbg = ""
-        #        for r in row:
-        #            dbg += str(r)
-        #            dbg += " | "
-        #        debug("delete_rec_iter acting on: %s"%dbg,5)
-        #        # this seems to be necessary to prevent
-        #        # a segfault on removing the last iter
-        #        self.rectree.set_model(empty_model)
-        #        row.model.remove(row.iter)
-        #        self.rectree.set_model(self.rmodel_sortable)
-        #        debug('delete_rec_iter removed recipe with ID %s'%rec.id,2)
-        #
-        #debug('delete_rec_iter couldn\'t find recipe with ID %s'%rec.id,1)
+        self.redo_search()
 
-    def update_rec_iter (self, rec):
-        # if r is already in our treeModel, we update values.
-        # otherwise, we add it.
-        debug("update_rec_iter called",5)
-        for row in self.rmodel:
-            recobj = row[0]
-            if recobj.id == rec.id:
-                row[0]=rec
-                if rec.thumb:
-                    row[1]=get_pixbuf_from_jpg(rec.thumb)
-                else:
-                    row[1]=None                
-                n = 2
-                for c in self.rtcols:
-                    row[n]=str(getattr(rec,c))
-                    n+=1
-                debug("update_rec_iter done!, id=%s"%rec.id,)
-                return
-        else: self.new_rec_iter(rec)
-    
-    def set_iter_from_rec (self, r, iter, visible=True):
-        """Handed a rec and an iter, set the row values appropriately."""
-        self.rmodel.set_value(iter, 0, r)
-        if r.thumb:
-            self.rmodel.set_value(iter, 1, get_pixbuf_from_jpg(r.thumb))
-        else:
-            self.rmodel.set_value(iter, 1, None)
-        n = 2
-        for c in self.rtcols:
-            debug("setting column %s (n=%s)"%(c,n),5)
-            self.rmodel.set_value(iter, n, str(getattr(r,c)))
-            n += 1
-
-    def create_rmodel (self, rview):
-        debug("create_rmodel (self, rview):",5)
-        # we start with our recipe object and our image
-        mod = [gobject.TYPE_PYOBJECT,gtk.gdk.Pixbuf]
-        # we add columns for all attributes...
-        for n in self.rtcols:
-            mod.append(gobject.TYPE_STRING)
-        self.rmodel = apply(gtk.TreeStore,mod)
-        # now we add our recipes
-        for r in rview:
-            self.new_rec_iter(r)
+    def update_rec_iter (self, rec): self.rmodel.update_recipe(rec)
 
     def recTreeDeleteRecCB (self, *args):
         """Make a watch show up (this can be slow
@@ -537,8 +518,8 @@ class RecGui (RecIndex):
             if rc.edited:
                 rc.widget.present()
                 if not de.getBoolean(
-                    label=_('Delete %s?')%rec.title,
-                    sublabel=_('You have unsaved changes to %s. Are you sure you want to delete?')%rec.title,
+                    label=_('Delete %s?'),
+                    sublabel=_('You have unsaved changes to %s. Are you sure you want to delete?'),
                     custom_yes=gtk.STOCK_DELETE,
                     custom_no=gtk.STOCK_CANCEL,
                     cancel=False):
@@ -554,9 +535,10 @@ class RecGui (RecIndex):
         if cancelled:
             for c in cancelled: recs.remove(c)
         self.rd.undoable_delete_recs(
-            recs,
+            [self.rd.get_rec(r.id) for r in recs],
             self.history,
-            make_visible=lambda *args: self.rmodel_filter.refilter())
+            make_visible=lambda *args: self.redo_search()
+            )
         self.set_reccount()
         self.message(_("Deleted ") + string.join([r.title for r in recs],', '))
 
@@ -578,32 +560,15 @@ class RecGui (RecIndex):
                 msg += "\n<i>%s</i>"%r.title
         else:
             msg = _("Are you sure you want to delete the %s selected recipes?")%len(recs)
-            tree = te.QuickTree(map(lambda r: r.title, recs))
+            tree = te.QuickTree([r.title for r in recs])
             expander = [_("See recipes"),tree]
         if de.getBoolean(parent=self.app,label=bigmsg,sublabel=msg,expander=expander):
-            debug('deleting iters... ',0)
-            #print 'deleting iters...'
-            self.doing_multiple_deletions=True
-            self.iters_to_remove=[]
-            for p in paths:
-                # we remove by hand...
-                iter = model.get_iter(p)
-                child = model.convert_iter_to_child_iter(None, iter)
-                grandchild = self.recTrash.rmodel_filter.convert_iter_to_child_iter(child)
-                self.iters_to_remove.append(grandchild)
-            self.iters_to_remove.sort()
-            self.iters_to_remove.reverse()
-            trees = [(self.rectree,self.rectree.get_model())]
-            if hasattr(self,'recTrash'): trees.append((self.recTrash.rectree,self.recTrash.rectree.get_model()))
-            for tree,mod in trees:
-                tree.set_model(empty_model)
-            for i in self.iters_to_remove:
-                self.recTrash.rmodel.remove(i)
-            for tree,mod in trees: tree.set_model(mod)
+            self.redo_search() # update all
+
             def show_progress (t):
                 gt.gtk_enter()
                 self.show_progress_dialog(t,
-                                          prog_dialog_kwargs={'label':'Deleting recipes'},
+                                          progress_dialog_kwargs={'label':'Deleting recipes'},
                                           message=_('Deletion paused'), stop_message=_("Stop deletion"))
                 gt.gtk_leave()
             def save_delete_hooks (t):
@@ -646,9 +611,16 @@ class RecGui (RecIndex):
             w.destroy()
             self.updateViewMenu()
         if hasattr(rec,'id') and rec.id:
-            titl = rec.title
-            debug('deleting recipe',5)
-            self.rd.delete_rec(rec.id)
+            if rec:
+                titl = rec.title
+                debug('deleting recipe %s'%rec.title,1)
+                # try a workaround to segfaults -- grab rec anew from ID.
+                dbrec = self.rd.get_rec(rec.id)
+                if dbrec:
+                    self.rd.delete_rec(dbrec)
+                else:
+                    print 'wtf?!?',rec,':',rec.id,' not real?'
+            else: debug('no recipe to delete!?!',1)
             if not self.doing_multiple_deletions:
                 gt.gtk_enter()
                 self.message(_("Deleted recipe %s")%titl)
@@ -674,17 +646,18 @@ class RecGui (RecIndex):
         if not hasattr(self,'recTrash'):
             self.recTrash = RecTrash(self.rd,self.rg)
         else:
-            self.recTrash.window.show()
+            self.recTrash.show()
 
     def empty_trash (self, *args):
         recs = self.rd.rview.select(deleted=True)
         self.rg.recTreePurge(recs)
         if hasattr(self,'recTrash'):
-            self.recTrash.rmodel_filter.refilter()
+            self.recTrash.rmodel.update_tree()
 
     def print_recs (self, *args):
         debug('printing recipes',3)
         recs = self.recTreeSelectedRecs()
+        gt.gtk_leave()
         printer.RecRenderer(self.rd, recs,
                             dialog_title=gettext.ngettext('Print %s recipe',
                                                           'Print %s recipes',
@@ -692,10 +665,15 @@ class RecGui (RecIndex):
                             dialog_parent = self.app,
                             change_units = self.prefs.get('readableUnits',True)
                             )
-    
-    def popup_rmenu (self, *args):
+        gt.gtk_enter()
+
+    def popup_rmenu (self, event):
         debug("popup_rmenu (self, *args):",5)
-        self.pop.popup(None,None,None,0,0)
+        if hasattr(event,'button') and hasattr(event,'time'):
+            self.pop.popup(None,None,None,event.button,event.time)
+        else:
+            self.pop.popup(None,None,None,0,0)
+            return True
 
     def newRecCard (self, *args):
         self.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
@@ -721,6 +699,7 @@ class RecGui (RecIndex):
                 self.app.window.set_cursor(None)
             self.app.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
             gobject.idle_add(show)
+
     def recTreeSelectRec (self, *args):
         debug("recTreeSelectRec (self, *args):",5)
         for rec in self.recTreeSelectedRecs():
@@ -731,7 +710,7 @@ class RecGui (RecIndex):
         rr=self.recTreeSelectedRecs()
         #r = self.recTreeSelectedRec()
         for r in rr:
-            if r.servings and r.servings != "None" and float(r.servings):
+            if r.servings and r.servings != "None":
                 debug("servings=%s"%r.servings,5)
                 serv = de.getNumber(default=float(r.servings),
                                     label=_("Number of servings of %s to shop for")%r.title,
@@ -796,7 +775,14 @@ class RecGui (RecIndex):
             cancel=False):
             self.rg.rd.clear_remembered_optional_ings() #without an arg, this clears all.
 
-    def exportg (self, *args):        
+    def export_selected_recs (self, *args): self.exportg(export_all=False)
+    def export_all_recs (self, *args): self.exportg(export_all=True)
+
+    def exportg (self, export_all=False):
+        """If all is false, export only selected recipes.
+
+        If all is True, export all recipes.
+        """
         if not use_threads and self.lock.locked_lock():
             de.show_message(
                 parent=self.app.get_toplevel(),
@@ -819,9 +805,10 @@ class RecGui (RecIndex):
             if exporters.exporter_dict.has_key(exp_type):
                 myexp = exporters.exporter_dict[exp_type]
                 pd_args={'label':myexp['label'],'sublabel':myexp['sublabel']%{'file':file}}
+                if export_all: recs = self.rd.fetch_all(self.rd.rview,deleted=False)
+                else: recs = self.recTreeSelectedRecs()
                 expClass = myexp['mult_exporter']({'rd':self.rd,
-                                                   #'rv':self.rd.rview.select(deleted=False),
-                                                   'rv':self.recTreeSelectedRecs(),
+                                                   'rv': recs,
                                                    'conv':self.conv,
                                                    'prog':self.set_progress_thr,
                                                    'file':file})
@@ -831,7 +818,7 @@ class RecGui (RecIndex):
                     debug('showing pause button',1)
                     gt.gtk_enter()
                     self.show_progress_dialog(t,message=_('Export Paused'),stop_message=_("Stop export"),
-                                              prog_dialog_kwargs=pd_args,
+                                              progress_dialog_kwargs=pd_args,
                                               )
                     gt.gtk_leave()
                 pre_hooks = [show_progress]
@@ -871,9 +858,6 @@ class RecGui (RecIndex):
             return
         import importers.html_importer
         sublabel = _('Enter the URL of a recipe archive or recipe website.')
-        sublabel += "\n"
-        sublabel += _('Gourmet can download a recipe in any supported recipe archive format or can import recipes from the following websites:')
-        sublabel += " %s"%", ".join(importers.html_importer.SUPPORTED_URLS.keys())
         url = de.getEntry(label=_('Enter website address.'),
                           sublabel=sublabel,
                           entryLabel=_('Enter URL: '),
@@ -884,7 +868,7 @@ class RecGui (RecIndex):
         if url.find('//')<0:
             url = 'http://'+url
         self.show_progress_dialog(None,
-                                  prog_dialog_kwargs={'label':_('Importing recipe from %s')%url,
+                                  progress_dialog_kwargs={'label':_('Importing recipe from %s')%url,
                                                       'stop':False,
                                                       'pause':False,})
         try:
@@ -913,7 +897,6 @@ class RecGui (RecIndex):
                             sublabel=sublabel,
                             message_type=gtk.MESSAGE_ERROR)
             self.hide_progress_dialog()
-            raise
         except BadZipfile:
             de.show_message(label=_('Unable to unzip'),
                             sublabel=_('Gourmet is unable to unzip the file %s')%url,
@@ -934,9 +917,7 @@ class RecGui (RecIndex):
                 sublabel=_('Are you sure %(url)s points to a page with a recipe on it?')%locals()
                 )
             raise
-        #else:            
-        #    #self.lsrch=['',''] #reset our treeview...
-        #    #self.search()
+        self.make_rec_visible()
         
     def importg (self, *args):
         if not use_threads and self.lock.locked_lock():
@@ -955,6 +936,7 @@ class RecGui (RecIndex):
         if ifiles:
             self.prefs['rec_import_directory']=os.path.split(ifiles[0])[0]
             self.import_multiple_files(ifiles)
+        self.make_rec_visible()
             
     def prepare_import_classes (self, files):
         """Handed multiple import files, prepare to import.
@@ -1108,8 +1090,7 @@ class RecGui (RecIndex):
         debug('hooks: %s'%self.rd.add_hooks,1)
         self.wait_to_filter=False
         gt.gtk_enter()
-        self.lsrch=['','']
-        self.search()
+        self.redo_search()
         gt.gtk_leave()
 
     def after_dialog_offer_url (self, linktype, file):
@@ -1121,7 +1102,8 @@ class RecGui (RecIndex):
         sublabel = _("Exported %s to %s"%(linktype,file))
         def offerer (t):
             if t.completed:
-                self.idle_offer_url(label, sublabel, url, True)
+                #self.idle_offer_url(label, sublabel, url, True)
+                self.offer_url(label, sublabel, url, True)
         return offerer
 
     def idle_offer_url (self, label, sublabl, url, from_thread):
@@ -1130,13 +1112,13 @@ class RecGui (RecIndex):
         gobject.idle_add(lambda *args: self.offer_url(label,sublabl,url,True))
         if from_thread:
             gt.gtk_leave()
-            
+
     def offer_url (self, label, sublabel, url, from_thread=False):
         if from_thread:
             gt.gtk_enter()
-        if hasattr(self,'prog_dialog'):
+        if hasattr(self,'progress_dialog'):
             self.hide_progress_dialog()            
-        d=de.messageDialog(label=label,
+        d=de.MessageDialog(label=label,
                            sublabel=sublabel,
                            cancel=False
                            )
@@ -1182,21 +1164,12 @@ class RecGui (RecIndex):
         gt.gtk_enter()
         self.set_reccount()
         gt.gtk_leave()
-        
+
     def set_progress_thr (self, prog, message=_("Importing...")):
         debug("set_progress_thr (self, %s,%s)"%(prog,message),1)
         gt.gtk_enter()
-        if prog < 0:
-            self.prog.pulse()
-        else:
-            if prog > 1.0:
-                print 'setting ',prog,'to 1'
-                prog = 1.0
-            self.prog.set_fraction(prog)
-        #self.stat.push(self.contid,"%s %s"%(len(self.rd.rview),message))
-        self.prog.set_text(message)
-        if prog==1:
-            self.prog_dialog.set_response_sensitive(gtk.RESPONSE_OK,True)
+        if hasattr(self,'progress_dialog'):
+            self.progress_dialog.set_progress(prog,message)
         gt.gtk_leave()
 
     def register_col_dialog (self, *args):
@@ -1231,6 +1204,9 @@ class RecGui (RecIndex):
 
 
 class RecTrash (RecIndex):
+
+    default_searches = [{'column':'deleted','operator':'=','search':True}]
+    
     def __init__ (self, rd, rg):
         self.rg = rg
         self.rmodel = self.rg.rmodel
@@ -1251,34 +1227,32 @@ class RecTrash (RecIndex):
             'undeleteRecs':self.recTreeUndeleteSelectedRecs,
             'ok':self.dismiss,
             })
-        RecIndex.__init__(self, self.rg.rmodel, self.glade, self.rg.rd, self.rg)
-        self.rmodel_filter.refilter()
+        RecIndex.__init__(self, self.glade, self.rg.rd, self.rg)
 
     def dismiss (self, *args):
         self.window.hide()
         return True
     
-    def setup_search_views (self):
-        self.lsrch = ["",""]
-        self.lsrchvw = self.rd.rview.select(deleted=True)
-        self.searchvw = self.rd.rview.select(deleted=True)
+    def show (self, *args, **kwargs):
+        self.rmodel.update_tree()
+        self.window.show(*args,**kwargs)
 
-    def visibility_fun (self, model, iter):
-        if (model.get_value(iter,0) and
-            model.get_value(iter,0).deleted and
-            model.get_value(iter, 0).id in self.visible):
-            return True
-        else:
-            return False
+    def setup_search_views (self):
+        self.last_search = ["",""]
+        self.rvw = self.rd.fetch_all(self.rd.rview,deleted=True)
+        self.searches = self.default_searches
 
     def recTreeUndeleteSelectedRecs (self, *args):
         mod,rr = self.rectree.get_selection().get_selected_rows()
         recs = [mod[path][0] for path in rr]
+        msg = ''
         for r in recs:
-            r.deleted = False
-        self.rmodel_filter.refilter()
-        self.rg.rmodel_filter.refilter()
-        self.rg.message(_('Undeleted recipes ') + string.join([r.title for r in recs],", "))
+            msg += r.title + ', '
+            self.rg.rd.modify_rec(r,{'deleted':False})
+        if msg: msg = msg[0:-2] # cut off the last comma
+        self.rmodel.update_tree()
+        self.rg.redo_search()
+        self.rg.message(_('Undeleted recipes ') + msg)
 
     def recTreePurgeSelectedRecs (self, *args):
         if not use_threads and self.rg.lock.locked_lock():
@@ -1293,9 +1267,6 @@ class RecTrash (RecIndex):
         recs = map(lambda path: mod[path][0],rr)
         self.rg.recTreePurge(recs,rr,mod)
 
-#    def showShopEditor (self, *args):
-#        se=shopEditor.ShopEditor(rd=self.rd,rg=self)
-
 def set_accel_paths (glade, widgets, base='<main>'):
     """A convenience function. Hand us a function and set accel
     paths based on it."""
@@ -1314,7 +1285,6 @@ def set_path_for_menuitem (mi, base='<main>'):
         accelLab = mi.get_children()[0]
         l=accelLab.get_label().replace('_','')
         path = base + '/' + l
-        #print 'setting path ',path
         mi.set_accel_path(path)
     sm = mi.get_submenu()
     if sm:
@@ -1376,10 +1346,17 @@ def startGUI ():
 if __name__ == '__main__':
     if os.name!='nt':
         import profile, tempfile,os.path
-        profi = os.path.join(tempfile.tempdir,'GOURMET_PROFILE')
-        profile.run('startGUI()',profi)
-        import pstats
-        p=pstats.Stats(profi)
-        p.strip_dirs().sort_stats('cumulative').print_stats()
+        import hotshot, hotshot.stats
+        #profi = os.path.join(tempfile.tempdir,'GOURMET_PROFILE')
+        prof = hotshot.Profile(os.path.join(tempfile.tempdir,'GOURMET_HOTSHOT_PROFILE'))
+        prof.runcall(startGUI)
+        stats = hotshot.stats.load(os.path.join(tempfile.tempdir,'GOURMET_HOTSHOT_PROFILE'))
+        stats.strip_dirs()
+        stats.sort_stats('time','calls')
+        stats.print_stats()
+        #profile.run('startGUI()',profi)
+        #import pstats
+        #p=pstats.Stats(profi)
+        #p.strip_dirs().sort_stats('cumulative').print_stats()
     else:
         startGUI()
