@@ -2,9 +2,46 @@
 import re, locale
 from defaults import lang as defaults
 from gettext import gettext as _
+from gettext import ngettext
 from gdebug import *
 
+FRACTIONS_ALL = 1
+FRACTIONS_NORMAL = 0
+FRACTIONS_ASCII = -1
+
 class converter:
+
+    unit_to_seconds = {
+    'seconds':1,
+    'minutes':60,
+    'hours':60*60,
+    'days':24*60*60,
+    'weeks':7*24*60*60,
+    'months':31*24*60*60,
+    'years':365.25*24*60*60,
+    #'decades':10*365.25*24*60*60,
+    #'centuries':100*365.25*24*60*60,
+    #'millenia':1000*365.25*24*60*60,
+    }
+
+    # this is a bit of a hackish attempt to make a matcher for all
+    # plural forms of time units. We use range(5) since as far as I
+    # can see, enumerating the forms for 0 through 5 should give you
+    # all possible forms in all languages.
+    #
+    # See http://www.sbin.org/doc/glibc/libc_8.html
+    time_units = [('seconds',[ngettext('second','seconds',n) for n in range(5)]),
+                  # min. = abbreviation for minutes
+                  ('minutes',[_('min.')] + [ngettext('minute','minutes',n) for n in range(5)]),
+                  # hrs = abbreviation for hours
+                  ('hours',[_('hrs.')] + [ngettext('hour','hours',n) for n in range(5)]),
+                  ('days',[ngettext('day','days',n) for n in range(5)]),
+                  ('years',[ngettext('year','years',n) for n in range(5)]),
+                  #('decades',[ngettext('decade','decades',n) for n in range(5)]),
+                  #('centuries',[ngettext('century','centuries',n) for n in range(5)]),
+                  #('millenia',[ngettext('millenium','millenia',n) for n in range(5)]),
+                  ]
+    
     def __init__(self):
         self.create_conv_table()
         self.create_density_table()
@@ -15,8 +52,14 @@ class converter:
         self.cross_unit_dicts={'density':self.density_table}
         ## This allows for varied spellings of units to be entered.
         self.create_unit_dict()
+        self.add_time_table()
         self.build_converter_dictionary()
         self.build_converter_dictionary(self.v2m_table,density=True)
+        
+
+    def add_time_table (self):
+        for u,conv in self.unit_to_seconds.items():
+            self.conv_table[(u,'seconds')]=conv        
 
     def create_conv_table(self):
         self.conv_table = defaults.CONVERTER_TABLE.copy()
@@ -31,7 +74,16 @@ class converter:
         self.cross_unit_table = defaults.CROSS_UNIT_TABLE.copy()
 
     def create_unit_dict(self):
-        self.units = defaults.UNITS
+        self.units = defaults.UNITS[0:]
+        for u,alts in self.time_units:
+            lst = []
+            for a in alts:
+                if not a in lst: lst.append(a)
+                if not a.title() in lst: lst.append(a.title())
+                if not a.capitalize() in lst: lst.append(a.capitalize())
+                if not a.upper() in lst: lst.append(a.upper())
+                if not a.lower() in lst: lst.append(a.lower())
+            self.units.append((u,lst))
         self.unit_dict={}
         for itm in self.units:
             key = itm[0]
@@ -42,6 +94,7 @@ class converter:
 
     def build_converter_dictionary (self, table=None, density=False):
         # first, make a list of all units in our dictionaries
+        #print 'build converter dictionary!'
         if not density:
             convert = self.convert_simple
         else:
@@ -50,11 +103,16 @@ class converter:
                 if self.convert_simple(u1,u2): return None
                 return self.convert_w_density(u1,u2,density=1)
         units = []
-        if not table: table=self.conv_table
-        for u1,u2 in table.keys():
+        if not table:
+            table=self.conv_table
+        #else:
+            #print "We were handed a table: ",table
+        for u1,u2 in filter(lambda x: len(x)==2, table.keys()):
             if u1 not in units: units.append(u1)
-            if u2 not in units: units.append(u2)            
+            if u2 not in units: units.append(u2)
+        #print 'done looping through list'
         for u in units:
+            #print 'grabbing possible conversions for ',u
             debug('unit=%s'%u)
             d=self.possible_conversions(u,dict=table)
             to_expand = d.keys()
@@ -139,7 +197,7 @@ class converter:
                 ret[k] = conv * mult
         return ret
 
-    def converter (self, u1, u2, item=None):
+    def converter (self, u1, u2, item=None, density=None):
         ## Just a front end to convert_fancy that looks up units
         if self.unit_dict.has_key(u1):
             unit1 = self.unit_dict[u1]
@@ -150,7 +208,7 @@ class converter:
         else:
             unit2 = u2
         ## provide some kind of item lookup?
-        return self.convert_fancy(unit1, unit2, item=item)
+        return self.convert_fancy(unit1, unit2, item=item, density=density)
 
     def convert_fancy (self, u1, u2, item=None, density=None):
         simple = self.convert_simple(u1,u2,self.conv_table)
@@ -386,9 +444,112 @@ class converter:
                 nstring += float_to_frac(num[1],approx=approx).strip()
         else:
             nstring = float_to_frac(num,approx=approx)
-        return "%s %s" %(nstring, un)
+        if un:
+            return "%s %s" %(nstring, un)
+        else:
+            return "%s"%nstring
 
+    def timestring_to_seconds (self, timestring):
+        """Take a timestring and parse it into seconds.
+
+        We assume numbers come before time units - surely this will
+        break some languages(?). We'll build the parameter in when the
+        time comes...
+        """
+        numbers = []
+        for match in NUMBER_FINDER.finditer(timestring):
+            if numbers: numbers[-1].append(match.start())
+            numbers.append([match.start(),match.end()])
+        if numbers: numbers[-1].append(None)
+        print numbers
+        secs = 0
+        for num_start,num_end,section_end in numbers:
+            num = frac_to_float(timestring[num_start:num_end])
+            unit = timestring[num_end:section_end].strip()
+            if self.unit_dict.has_key(unit):
+                conv = self.converter(unit,'seconds')
+                if conv:
+                    secs += num * conv
+        return secs
+
+    def timestring_to_seconds_old (self, timestring):
+        """Take a timestring and parse it into seconds.
+
+        This logic may be a little fragile for non-English languages.
+        """
+        words = re.split('[ \s,;]+',str(timestring))
+        seconds = 0
+        num = []
+        for n,w in enumerate(words):
+            if NUMBER_MATCHER.match(w): num.append(w)
+            elif num and self.unit_dict.has_key(w):                
+                conv = self.converter(w,'seconds')
+                if conv:
+                    n = frac_to_float(" ".join(num))
+                    if n: seconds += n * conv
+                    num = []
+        if seconds: return seconds
     
+
+# Each of our time formatting functions takes two arguments, which
+# allows us to handle fractions in the outside world
+time_formatters = {
+    #'millennia':lambda decades: ngettext("millenium","millenia",round(decades))
+    #'centuries':lambda decades: ngettext("century","centuries",round(decades))
+    #'decades':lambda decades: ngettext("decade","decades",round(decades))
+    'years':lambda years: ngettext("year","years",years),
+    'months':lambda months: ngettext("month","months",months),
+    'weeks':lambda weeks: ngettext("week","weeks",weeks),
+    'days':lambda days: ngettext("day","days",days),
+    'hours':lambda hours: ngettext("hour","hours",hours),
+    'minutes':lambda minutes: ngettext("minute","minutes",minutes),
+    'seconds':lambda seconds: ngettext("second","seconds",seconds),
+    }    
+
+def seconds_to_timestring (time, round_at=None, fractions=FRACTIONS_NORMAL):
+    time = int(time)
+    time_strings = []
+    units = converter.unit_to_seconds.items()
+    units.sort(lambda a,b: a[1]<b[1] and 1 or a[1]>b[1] and -1 or 0)
+    for unit,divisor in units:  
+        time_covered = time / int(divisor)
+        # special case hours, which we English speakers anyway are
+        # used to hearing in 1/2s -- i.e. 1/2 hour is better than 30
+        # minutes.
+        add_half = 0
+        if time % divisor == .5*divisor and not time_strings:
+            add_half = 0.5
+            time_covered += 0.5
+        if time_covered or add_half:
+            #print time_covered,'(rounds to ',round(time_covered),')'
+            if round_at and len(time_strings)+1>=round_at:
+                if not add_half: time_covered = int(round(float(time)/divisor))
+                time_strings.append(" ".join([
+                    float_to_frac(time_covered,fractions=fractions),
+                    # round because 1/2 = 1 as far as plural formas are concerned
+                    time_formatters[unit](round(time_covered)) 
+                    ])
+                                    )
+                break
+            else:
+                time_strings.append(" ".join([
+                    float_to_frac(time_covered,fractions=fractions),
+                    # round because 1/2 = 1 as far as plural forms are concerned
+                    time_formatters[unit](round(time_covered))
+                    ]))
+                time = time - time_covered * divisor
+                if time==0: break
+    if len(time_strings)>2:
+        # Translators... this is a messay way of concatenating
+        # lists. In English we do lists this way: 1, 2, 3, 4, 5
+        # and 6. This set-up allows for variations of this system only.
+        # You can of course make your language only use commas or
+        # ands or spaces or whatever you like by translating both
+        # ", " and " and " with the same string.
+        return _(" and ").join([_(", ").join(time_strings[0:-1]),time_strings[-1]])
+    else:
+        return _(" ").join(time_strings)    
+
 def integerp (num, approx=0.01):
     """approx can be a decimal that is a guide to rounding.
     That is, if approx is 0.001, then we will consider
@@ -412,6 +573,22 @@ def integerp (num, approx=0.01):
 # want to limit special fractions to these and use straight-up ascii
 # fractions otherwise. These fractions must also be keys in
 # NUM_TO_FRACTIONS
+
+NUMBER_WORDS = {}
+if hasattr(defaults,'NUMBERS'):
+    for n,words in defaults.NUMBERS.items():
+        for w in words:
+            NUMBER_WORDS[w] = n
+all_number_words = NUMBER_WORDS.keys()
+all_number_words.sort(
+    lambda x,y: ((len(y)>len(x) and 1) or (len(x)>len(y) and -1) or 0)
+    )
+
+NUMBER_WORD_REGEXP = '|'.join(all_number_words).replace(' ','\s+')
+FRACTION_WORD_REGEXP = '|'.join(filter(lambda n: NUMBER_WORDS[n]<1.0,
+                                       all_number_words)
+                                ).replace(' ','\s+')
+
 NORMAL_FRACTIONS = [(1,2),(1,4),(3,4)] 
 
 NUM_TO_FRACTIONS = {
@@ -480,9 +657,16 @@ NUMBER_REGEXP = "[\d.,"
 for k in UNICODE_FRACTIONS.keys(): NUMBER_REGEXP+=k
 NUMBER_START_REGEXP = NUMBER_REGEXP + ']'
 NUMBER_END_REGEXP = NUMBER_REGEXP + SLASH
+NUMBER_END_NO_RANGE_REGEXP = NUMBER_END_REGEXP + " /]"
 NUMBER_END_REGEXP += " /-"
 NUMBER_END_REGEXP += "]"
-NUMBER_REGEXP = "("+NUMBER_START_REGEXP+"+"+NUMBER_END_REGEXP+"*"+")"
+NUMBER_REGEXP = "("+NUMBER_START_REGEXP+NUMBER_END_REGEXP+"*"
+if NUMBER_WORD_REGEXP:
+     NUMBER_REGEXP = NUMBER_REGEXP + '|' + NUMBER_WORD_REGEXP + ')'
+     NUMBER_NO_RANGE_REGEXP = '(' + NUMBER_START_REGEXP + '+|' + NUMBER_WORD_REGEXP + ')'
+else:
+    NUMBER_REGEXP = NUMBER_REGEXP + ")"
+    NUMBER_NO_RANGE_REGEXP = NUMBER_START_REGEXP + '+'
 NUMBER_MATCHER = re.compile("^%s$"%NUMBER_REGEXP,re.UNICODE)
 
 UNICODE_FRACTION_REGEXP = "[" + "".join(UNICODE_FRACTIONS.keys()) + "]"
@@ -493,8 +677,24 @@ DIVISOR_REGEXP = "[0-9" + "".join(SUB_DICT.values()) + "]+"
 FRACTION_REGEXP = "(" + UNICODE_FRACTION_REGEXP + "|" + DIVIDEND_REGEXP + \
                           SLASH_REGEXP + DIVISOR_REGEXP + ")"
 
+AND_REGEXP = "(\s+%s\s+|\s*[&+]\s*|\s+)"%_('and')
+
 # Match a fraction
-FRACTION_MATCHER = re.compile("([0-9]+\s+)?%s"%FRACTION_REGEXP,re.UNICODE)
+if NUMBER_WORD_REGEXP:
+    NUM_AND_FRACTION_REGEXP = "((?P<int>%s+|%s)%s)?(?P<frac>(%s|%s))"%(NUMBER_START_REGEXP,
+                                                                       NUMBER_WORD_REGEXP,
+                                                                       AND_REGEXP,
+                                                                       FRACTION_REGEXP,
+                                                                       FRACTION_WORD_REGEXP
+                                                                       )
+    
+else:
+    NUM_AND_FRACTION_REGEXP = re.compile("((?P<int>%s)+\s+)?(?P<frac>%s)"%(NUMBER_START_REGEXP,FRACTION_REGEXP),re.UNICODE)
+
+FRACTION_MATCHER = re.compile(NUM_AND_FRACTION_REGEXP,re.UNICODE)
+
+NUMBER_FINDER_REGEXP = "(%(NUM_AND_FRACTION_REGEXP)s|%(NUMBER_NO_RANGE_REGEXP)s)(?=($| |[\W]))"%locals()
+NUMBER_FINDER = re.compile(NUMBER_FINDER_REGEXP,re.UNICODE)
 
 # Note: the order matters on this range regular expression in order
 # for it to properly split things like 1 - to - 3, which really do
@@ -507,22 +707,27 @@ RANGE_MATCHER = re.compile(RANGE_REGEXP[1:-1]) # no parens for this one
 # ingredients (e.g. 1 cup milk) that get misparsed by other ingredient
 # parsers. This is often necessary because formats like mealmaster and
 # mastercook are rarely actually followed.
-ING_MATCHER = re.compile("""
+NUMBER_FINDER_REGEXP2 = NUMBER_FINDER_REGEXP.replace('int','int2').replace('frac','frac2')
+ING_MATCHER_REGEXP = """
  \s* # opening whitespace
- (
- %(NUMBER_START_REGEXP)s+ # a number
- %(NUMBER_END_REGEXP)s* # possibly a long/complex number
- %(RANGE_REGEXP)s? # a possible range delimiter
- %(NUMBER_END_REGEXP)s* # and more numbers
+ (?P<amount>
+ %(NUMBER_FINDER_REGEXP)s # a number
+ \s* # Extra whitespace
+ (%(RANGE_REGEXP)s # a possible range delimiter
+ \s* #More extra whitespace
+ %(NUMBER_FINDER_REGEXP2)s)? # and more numbers
  )? # and of course no number is possible
- (\s*[\w.]+\s+)? # a unit
- (.*?)$ # and the rest of our stuff...
- """%locals(),
+ \s* # Whitespace between number and unit
+ (?P<unit>\s*[\w.]+\s+)? # a unit
+ (?P<item>.*?)$ # and the rest of our stuff...
+ """%locals()
+
+ING_MATCHER = re.compile(ING_MATCHER_REGEXP,
                          re.VERBOSE|re.UNICODE)
 
-ING_MATCHER_AMT_GROUP = 0
-ING_MATCHER_UNIT_GROUP = 2
-ING_MATCHER_ITEM_GROUP = 3
+ING_MATCHER_AMT_GROUP = 'amount'
+ING_MATCHER_UNIT_GROUP = 'unit'
+ING_MATCHER_ITEM_GROUP = 'item'
 
 def convert_fractions_to_ascii (s):
     """Convert all unicode-like fractions in string S with their ASCII equivalents"""
@@ -534,10 +739,6 @@ def convert_fractions_to_ascii (s):
     s=re.sub(SLASH,'/',s)
     return s
 
-FRACTIONS_ALL = 1
-FRACTIONS_NORMAL = 0
-FRACTIONS_ASCII = -1
-
 def fractify (decimal, divisor, approx=0.01, fractions=FRACTIONS_NORMAL):
     """Return fraction equivalent of decimal using divisor
 
@@ -548,7 +749,7 @@ def fractify (decimal, divisor, approx=0.01, fractions=FRACTIONS_NORMAL):
     if dividend:
         if fractions==FRACTIONS_ASCII:
             return "%s/%s"%(dividend,divisor)
-        elif fractions==FRACTIONS_NORMAL:
+        elif fractions==FRACTIONS_ALL:
             # otherwise, we have to do nice unicode magic
             if NUM_TO_FRACTIONS.has_key((dividend,divisor)):
                 return NUM_TO_FRACTIONS[(dividend,divisor)]
@@ -571,7 +772,7 @@ def float_to_frac (n, d=[2,3,4,5,6,8,10,16],approx=0.01,fractions=FRACTIONS_NORM
     n=float(n)
     i = int(n)
     if i >= 1:
-        i="%s "%int(n)
+        i="%s"%int(n)
     else:
         i=""
     rem = n - int(n)
@@ -585,7 +786,7 @@ def float_to_frac (n, d=[2,3,4,5,6,8,10,16],approx=0.01,fractions=FRACTIONS_NORM
         for div in d:
             f = fractify(rem,div,approx=approx,fractions=fractions)
             if f:
-                return "%s%s"%(i,f)
+                return " ".join([i,f]).strip()
         if len("%s"%n) > 5:
             # use locale-specific formatting if fractions don't work
             return locale.format("%.2f",n,True) # format(formatstring, number, use_thousands_separator)
@@ -600,7 +801,10 @@ def float_string (s):
     thousands and thousands of things are rare.
     Also, we recognize items outside of our locale, since e.g. American might well be
     importing British recipes and viceversa.
-    """    
+    """
+    if NUMBER_WORDS.has_key(s.lower()):
+        print 'We have key',s.lower()
+        return NUMBER_WORDS[s.lower()]
     THOUSEP = locale.localeconv()['thousands_sep']
     DECSEP = locale.localeconv()['decimal_point']
     if s.count(',') > 1 and s.count('.') <= 1:
@@ -634,10 +838,14 @@ def frac_to_float (s):
     s = unicode(s)
     m=FRACTION_MATCHER.match(s)
     if m:
-        i,frac = m.groups()
-        if not i: i=0
+        i = m.group('int')
+        frac = m.group('frac')
+        if i: i=float_string(i)
+        else: i = 0
         if UNICODE_FRACTIONS.has_key(frac):
-            return float(i)+UNICODE_FRACTIONS[frac]
+            return i+UNICODE_FRACTIONS[frac]
+        elif NUMBER_WORDS.has_key(frac):
+            return i+NUMBER_WORDS[frac]
         else:
             n,d = SLASH_MATCHER.split(frac)
             n = SUP_DICT.get(n,n)
@@ -657,7 +865,7 @@ def frac_to_float (s):
 
 
 
-if __name__ == '__main__':
+if __name__ == '__main__' and False:
     class interactive:
         def __init__ (self):
             self.c = converter()
@@ -734,3 +942,6 @@ if __name__ == '__main__':
         def quit (self):
             import sys
             sys.exit()
+        
+    
+    #i=interactive()
