@@ -55,7 +55,7 @@ SYNONYMS=[
     ["chilli pepper","capsicum"],
     ["corn","sweetcorn","maise","sweet corn"],
     ["red bell pepper","red pepper"],
-    ["green pepper", "bell pepper","green bell pepper", "pepper"],
+    ["bell pepper, green", "green pepper", "bell pepper","green bell pepper", "pepper"],
     ["beet","beetroot"],
     ["collard greens","spring greens"],
     ["cilantro","coriander"],
@@ -77,9 +77,17 @@ SYNONYMS=[
     ["sapodilla", "chiku", "sapadilla", "snake fruit", "sawo"],
     ["soursop", "guanabana"],
     ['black cod','sablefish'],
-    ['patagonian toothfish','chilean sea bass'],
+    ['chilean sea bass','patagonian toothfish'],
+    ['flour, all purpose','flour, all-purpose','flour','white flour'],
+    ['sugar, white','sugar'],    
     ]
 
+# A DICTIONARY CONTAINING INGREDIENT KEYS AND NDBNO for the USDA
+# nutritional database. For these items, we will have nutritional
+# information by default.
+
+NUTRITIONAL_INFO = {}
+
 # a dictionary for ambiguous words.
 # key=ambiguous word, value=list of possible non-ambiguous terms
 #
@@ -120,7 +128,7 @@ INGREDIENT_DATA = [["alfalfa sprouts","alfalfa sprouts","produce"],
                    ["artichoke","artichoke","produce"],
                    ["arugula","arugula","produce"],
                    ["asparagus","asparagus","produce"],
-                   ["aubergine","aubergine","produce"],
+                   ["eggplant","eggplant","produce"],
                    ["avocado","avocado","produce"],
                    ["green beans","green beans","produce"],
                    ["azuki beans","azuki beans","produce"],
@@ -133,7 +141,7 @@ INGREDIENT_DATA = [["alfalfa sprouts","alfalfa sprouts","produce"],
                    ["green beans","green beans","produce"],
                    ["kidney beans","kidney beans","produce"],
                    ["lentils","lentils","produce"],
-                   ["lima bean or butter bean","lima bean or butter bean","produce"],
+                   ["lima bean","butter bean","produce"],
                    ["mung beans","mung beans","produce"],
                    ["navy beans","navy beans","produce"],
                    ["runner beans","runner beans","produce"],
@@ -175,8 +183,8 @@ INGREDIENT_DATA = [["alfalfa sprouts","alfalfa sprouts","produce"],
                    ["shallot","shallot","produce"],
                    ["scallion","scallion","produce"],
                    ["parsley","parsley","produce"],
-                   ["pepper","pepper","produce"],
-                   ["red bell pepper","red bell pepper","produce"],
+                   ["green pepper","bell pepper, green","produce"],
+                   ["red bell pepper","bell pepper, red","produce"],
                    ["chilli pepper","chilli pepper","produce"],
                    ["jalapeño pepper","pepper, jalapeño","produce"],
                    ["habanero pepper","pepper, habanero","produce"],
@@ -529,7 +537,8 @@ INGREDIENT_DATA = [["alfalfa sprouts","alfalfa sprouts","produce"],
                    ['eggs','eggs', 'dairy',],
                    ['salmon','salmon', 'seafood',],
                    ['lasagna pasta noodles','pasta, lasagna noodles', 'pastas',],
-                   ['all flour purpose','flour, all purpose', 'baking',],
+                   ['all purpose flour','flour, all purpose', 'baking',],
+                   ['flour','flour, all purpose','baking',],
                    ['ground cumin','cumin, ground', 'spices',],
                    ['cucumber','cucumber', 'produce',],
                    ['salsa','salsa', 'international',],
@@ -573,6 +582,7 @@ CONVERTER_TABLE = {
     ("l.", "cl."):100,
     ("l.", "dl."):10,
     ("oz.", "g."):28.35,
+    ("fl. oz.","tbs."):2,
     ("kg.", "g."):1000,
     ("g.", "mg."):1000,
     ("tsp.", "drop"):76,
@@ -634,21 +644,32 @@ UNITS = [["bucket" , ["bucket", "buckets", "bckt."]],
          ["grains", ["grain", "grains"]],
          ["dram", ["dram", "drams"]],
          ["drop",["drop"]],
-         ["tsp.", ["teaspoon","tsp", "tsp.","tea spoon", "tsps.", "teaspoons", "tea spoons", "Teaspoon", "Teaspoons","t","Ts.","Tsp.","Tsp"]],
-         ["tbs.", ["tablespoon","tbs", "tbsp", "tbs.", "tbsp.", "table spoon", "tbsps.", "tablespoons", "Tablespoon", "T", "Tbs.", "Tbsp", "Tbsp."]],
+         ["tsp.", ["teaspoon","tsp", "tsp.","tea spoon", "tsps.", "teaspoons", "tea spoons", "Teaspoon", "Teaspoons","t",'ts',"Ts.","Tsp.","Tsp"]],
+         ["tbs.", ["tablespoon","tbs", "tbsp", "tbs.", "tbsp.", "table spoon", "tbsps.", "tablespoons", "Tablespoon", "T",'tb',"Tbs.", "Tbsp", "Tbsp."]],
          ["lb.", [ "pound", "lb","lb.", "lbs.", "pounds"]],
          ["oz.", [ "ounce", "oz","ounces", "oz."]],
          ["c.", ["cup", "c.", "cups"]],
-         ["qt.", ["quart", "qt.", "quarts"]],
+         ["qt.", ["quart", "qt.", "quarts","Qt","Qt."]],
          ["pt.", ["pint", "pt.", "pints"]],
          ["gallon", ["gallon", "gallons","gal."]],
          ["ml.", ["mililiter","ml", "ml.","mililiters"]],
          ["cl.", ["centiliter","cl", "cl.", "centiliters"]],
          ["dl.", ["deciliter","dl", "dl.","deciliters"]],
-         ["l.", ["liter", "l.", "lit.", "liters"]],
-         ["g.", ["grams", "gram", "g."]],
+         ["l.", ["liter", "l.", "lit.", "liters",'l']],
+         ["g.", ["grams", "gram", "g.",'g']],
          ["mg.", ["miligram", "mg.", "mg", "miligrams"]],
-         ["kg.", ["kilogram","kg.", "kg",  "kilograms"]]
+         ["kg.", ["kilogram","kg.", "kg", "kilograms"]],
+         # These names aren't really convertible, but we want them to
+         # be recognized as units...
+         ['small',['small','sm','Small','sm.']],         
+         ['medium',['medium','med.','Medium']],
+         ['large',['large','Large','lg','lg.']],
+         ['clove',['clove','cloves','clv.']],
+         ['whole',['whole','wh.','whl']],
+         ['package',['pkg.','pkg','package','Pkg.','Package','pack']],
+         ['box',['Box','box','bx']],
+         ['can',['can','Can','cn','cn.']],
+         ['slices',['slice','slices']],
          ]
 
 METRIC_RANGE = (1,999)
@@ -682,6 +703,8 @@ UNIT_GROUPS = {
                        ('bushel',(1,None))]
     }
 
+
+
 # The units here need to correspond to the standard unit names defined
 # above in UNITS
 CROSS_UNIT_TABLE = {
@@ -696,7 +719,9 @@ CROSS_UNIT_TABLE = {
     ("tbs.", "oz."):['density',0.5],
     ("c.", "oz."):['density',8],
     ("pt.", "oz."):['density',16],
-    ("ml.", "g."):['density',1]}
+    ("ml.", "g."):['density',1],
+    ('oz.','fl. oz.'):['density',1],
+    }
 
 # The units here need to correspond to the standard unit names defined
 # in UNITS.  These are some core conversions from mass-to-volume,
@@ -714,3 +739,72 @@ VOL_TO_MASS_TABLE = {
     ("dl.", "kg.") : 0.1,
     ("dl.", "g.") : 100,    
     ("l.", "kg.") : 1}
+
+# TIME ABBREVIATIONS (this is new!)
+TIME_ABBREVIATIONS = {
+    'sec':'seconds',
+    'min':'minutes',
+    'hr':'hours'
+    }
+
+# These functions are rather important! Our goal is simply to
+# facilitate look ups -- if the user types in "tomatoes", we want to
+# find "tomato." Note that the strings these functions produce will
+# _never_ be shown to the user, so it's fine to generate nonsense
+# words as well as correct answers -- our goal is to generate a list
+# of possible hits rather than to get the plural/singular form "right".
+
+irregular_plurals={
+    "geese":"goose",
+    }
+import re
+two_digit_plural_matcher = re.compile('[szxo]es$')
+one_digit_plural_matcher = re.compile("[^s]s$")
+v_plural_matcher = re.compile('ves')
+
+def guess_singulars (s):
+    if len(s)<3: return []
+    rets = []
+    if irregular_plurals.has_key(s):
+        rets.append(irregular_plurals[s])
+    if two_digit_plural_matcher.search(s):
+        wrd=s[0:-2]
+        if not wrd in rets: rets.append(wrd)
+    if v_plural_matcher.search(s):
+        rets.append(s[0:-3]+'f')
+    if one_digit_plural_matcher.search(s): rets.append(s[0:-1])
+    return rets
+
+def guess_plurals (s):
+    if not s: return []
+    ret = [s+'s',s+'es']
+    if s[-1]=='f': ret.append(s[0:-1]+'ves')
+    return ret
+    
+IGNORE = ["and","with","of","for","cold","warm","finely","thinly","roughly","coarsely"]
+
+NUMBERS = {
+    (1.0/8):['eighth','an eigth'],
+    (1.0/4):['quarter','a quarter'],
+    (3.0/4):['three quarters'],
+    (2.0/3):['two thirds'],
+    (1.0/3):['third','a third'],
+    (1.0/2):['half','a half','one half'],
+    1:['an','a','one'],
+    2:['two','a couple','a couple of','a pair of'],
+    3:['three'],
+    4:['four'],
+    5:['five'],
+    6:['six'],
+    7:['seven'],
+    8:['eight'],
+    9:['nine'],
+    10:['ten'],
+    11:['eleven'],
+    12:['twelve','a dozen'],
+    20:['twenty'],
+    30:['thirty'],
+    40:['forty'],
+    50:['fifty'],
+    60:['sixty'],
+    }
