@@ -1105,3 +1105,34 @@ VOL_TO_MASS_TABLE = {
 #     ("dl","kg") : 0.1,
 #     ("dl","g") : 100,    
 #     ("l","kg") : 1}
+
+
+# These functions are rather important! Our goal is simply to
+# facilitate look ups -- if the user types in "tomatoes", we want to
+# find "tomato." Note that the strings these functions produce will
+# _never_ be shown to the user, so it's fine to generate nonsense
+# words as well as correct answers -- our goal is to generate a list
+# of possible hits rather than to get the plural/singular form "right".
+
+def guess_singulars (s):
+    """I don't really know French, but I'm going to guess it's
+    like English and Spanish with regard to plurals and try some basic
+    rules here.
+    """
+    if len(s)<3: return []
+    ret = []
+    if s[-1]=='s':
+        ret.append(s[0:-1]) # try chopping off the s
+        if s[-2]=='e':
+            ret.append(s[0:-2]) # try chopping off 'es'
+    return ret
+
+def guess_plurals (s): return [s+'s',s+'es']
+
+
+IGNORE=["et","avec","de","des","d'","pour","froid","froide","froids","froides",
+         "chaud","chaude","chauds","chaudes","finement","mince","minces",
+         "approximativement","grosso modo","vulgairement"]
+
+NUMBERS = {
+    }
