@@ -1,12 +1,16 @@
 #!/usr/bin/env python
 import gtk, gobject, os.path, optionTable, thumbnail, cb_extras, fnmatch
 import gglobals, re
+import pango
 import xml.sax.saxutils
 from gettext import gettext as _
 from gdebug import debug
 H_PADDING=12
 Y_PADDING=12
 
+class UserCancelledError (Exception):
+    pass
+
 def is_markup (s):
     try:
         pango.parse_markup(s,u'0')
@@ -14,7 +18,7 @@ def is_markup (s):
     except:
         return False
 
-class mDialog (gtk.Dialog):
+class ModalDialog (gtk.Dialog):
     def __init__ (self, default=None, title="", okay=True, label=False, sublabel=False, parent=None, cancel=True, modal=True, expander=None):
         """Our basic class. We allow for a label. Possibly an expander
         with extra information, and a simple Okay button.  The
@@ -53,8 +57,6 @@ class mDialog (gtk.Dialog):
     def setup_label (self, label):
         # we're going to add pango markup to our
         # label to make it bigger as per GNOME HIG
-        if not is_markup(label):
-            label = xml.sax.saxutils.escape(label)
         self.set_title(label)
         label = '<span weight="bold" size="larger">%s</span>'%label
         self.label = gtk.Label(label)
@@ -68,7 +70,6 @@ class mDialog (gtk.Dialog):
         self.label.show()
         
     def setup_sublabel (self,sublabel):
-        if not is_markup(sublabel): sublabel=xml.sax.saxutils.escape(sublabel)
         self.sublabel = gtk.Label(sublabel)
         self.sublabel.set_selectable(True)
         self.vbox.pack_start(self.sublabel, expand=False)
@@ -76,7 +77,7 @@ class mDialog (gtk.Dialog):
         self.sublabel.set_alignment(0,0)
         self.sublabel.set_justify(gtk.JUSTIFY_LEFT)
         self.sublabel.set_use_markup(True)
-        self.sublabel.set_line_wrap(True)
+        self.sublabel.set_line_wrap(gtk.WRAP_WORD)
         self.sublabel.show()
 
     def setup_buttons (self, cancel, okay):
@@ -110,7 +111,7 @@ class mDialog (gtk.Dialog):
         if type(item)==type(""):
             l=gtk.Label(item)
             l.set_selectable(True)
-            l.set_line_wrap(True)
+            l.set_line_wrap(gtk.WRAP_WORD)
             self.expander_vbox.pack_start(l,
                                           expand=False,
                                           fill=False)
@@ -134,14 +135,14 @@ class mDialog (gtk.Dialog):
         self.ret=None
         if self.modal: gtk.main_quit()
 
-class messageDialog (gtk.MessageDialog, mDialog):
+class MessageDialog (gtk.MessageDialog, ModalDialog):
 
     """A simple class for displaying messages to our users."""
     
     def __init__ (self, title="", default=None, okay=True, cancel=True, label=False, sublabel=False,
                   expander=None, message_type=gtk.MESSAGE_INFO, parent=None):
         self.message_type=message_type
-        mDialog.__init__(self, title=title, default=default, okay=okay, cancel=cancel, label=label, sublabel=sublabel, parent=parent, expander=expander)
+        ModalDialog.__init__(self, title=title, default=default, okay=okay, cancel=cancel, label=label, sublabel=sublabel, parent=parent, expander=expander)
 
     def setup_dialog (self, *args, **kwargs):
         kwargs['type']=self.message_type
@@ -151,24 +152,23 @@ class messageDialog (gtk.MessageDialog, mDialog):
 
     def setup_label (self, label):
         if not is_markup(label):
-            print 'Escaping',label
             label = xml.sax.saxutils.escape(label)
         label = '<span weight="bold" size="larger">%s</span>'%label
         self.set_markup(label)
 
     def setup_sublabel (self, sublabel):
-        if is_markup(sublabel):
-            self.format_secondary_markup(sublabel)
-        else:
-            self.format_secondary_text(sublabel)
+        #curtext = self.label.get_text()
+        #curtext += "\n%s"%sublabel
+        #self.label.set_text(xml.sax.saxutils.escape(curtext))
+        self.format_secondary_text(sublabel)
                   
-class numberDialog (mDialog):
+class NumberDialog (ModalDialog):
 
     """A dialog to get a number from our user."""
 
     def __init__(self,default=None,label=False,sublabel=False,step_incr=1,page_incr=10,digits=0,
                  min=0,max=10000, parent=None):
-        mDialog.__init__(self,default=default, parent=parent)
+        ModalDialog.__init__(self,default=default, parent=parent)
         self.hbox=gtk.HBox()
         self.vbox.add(self.hbox)
         #self.spinButton=gtk.SpinButton(climb_rate=climb_rate,digits=digits)
@@ -187,19 +187,24 @@ class numberDialog (mDialog):
         if label:
             self.label=gtk.Label(label)
             self.label.set_selectable(True)
-            self.label.set_line_wrap(True)
+            self.label.set_line_wrap(gtk.WRAP_WORD)
             self.label.set_padding(H_PADDING, Y_PADDING)
             self.hbox.add(self.label)
             self.label.show()
         self.hbox.add(self.spinButton)
         self.spinButton.get_adjustment().connect("value_changed",self.update_value)
+        self.spinButton.get_adjustment().connect("changed",self.update_value)
         self.spinButton.show()
+        self.spinButton.connect('activate',self.entry_activate_cb)
         self.hbox.show()                
         
     def update_value (self, *args):
         self.ret=self.spinButton.get_value()
 
-class entryDialog (mDialog):
+    def entry_activate_cb (self, *args):
+        self.okcb()
+
+class EntryDialog (ModalDialog):
 
     """A dialog to get some text from an Entry from our user."""
     
@@ -212,12 +217,12 @@ class entryDialog (mDialog):
                   visibility=True,
                   default_value=None,
                   default_character_width=None):
-        mDialog.__init__(self,default=default,parent=parent, label=label, sublabel=sublabel)
+        ModalDialog.__init__(self,default=default,parent=parent, label=label, sublabel=sublabel)
         self.hbox=gtk.HBox()
         self.vbox.add(self.hbox)
         if entryLabel:
             self.elabel=gtk.Label(entryLabel)
-            self.elabel.set_line_wrap(True)
+            self.elabel.set_line_wrap(gtk.WRAP_WORD)
             self.elabel.set_selectable(True)
             self.elabel.set_alignment(0,0)
             self.hbox.add(self.elabel)
@@ -255,43 +260,50 @@ class entryDialog (mDialog):
         if self.ret:
             self.okcb()
 
-class radioDialog (mDialog):
+class RadioDialog (ModalDialog):
+
 
     """A dialog to offer our user a choice between a few options."""
 
     def __init__ (self, default=None, label="Select Option", sublabel=None, options=[],
                   parent=None,expander=None,cancel=True):
-        mDialog.__init__(self, okay=True, label=label, sublabel=sublabel, parent=parent, expander=expander, cancel=cancel)
+        ModalDialog.__init__(self, okay=True, label=label, sublabel=sublabel, parent=parent, expander=expander, cancel=cancel)
         # defaults value is first value...
-        if options:
-            self.ret = options[0][1]
+        self.default = default
         self.setup_radio_buttons(options)
+        if options and default==None:
+            self.ret = options[0][1]
 
     def setup_radio_buttons (self,options):
         previous_radio = None
         self.buttons = []
         for label,value in options:
             rb = gtk.RadioButton(group=previous_radio, label=label, use_underline=True)
+            rb.connect('activate',self.okcb)
             self.vbox.add(rb)
             rb.show()
             rb.connect('toggled',self.toggle_cb,value)
             self.buttons.append(rb)
             previous_radio=rb
-        self.buttons[0].set_active(True)
-        self.widget_that_grabs_focus = self.buttons[0]
+            if self.default==value:
+                rb.set_active(True)
+                self.widget_that_grabs_focus = rb
+        if self.default==None:
+            self.buttons[0].set_active(True)
+            self.widget_that_grabs_focus = self.buttons[0]
 
     def toggle_cb (self, widget, value):
         if widget.get_active():
             self.ret = value
 
-class optionDialog (mDialog):
+class OptionDialog (ModalDialog):
 
     """A dialog to offer our user a choice between options using an option menu"""
     
     def __init__ (self, default=None, label="Select Option", sublabel=None, options=[], parent=None, expander=None, cancel=True):
         """Options can be a simple option or can be a tuple or a list
         where the first item is the label and the second the value"""
-        mDialog.__init__(self, okay=True, label=label, sublabel=sublabel, parent=parent, expander=expander, cancel=cancel)
+        ModalDialog.__init__(self, okay=True, label=label, sublabel=sublabel, parent=parent, expander=expander, cancel=cancel)
         self.menucb = self.get_option        
         self.optdic={}
         self.menu = gtk.Menu()
@@ -324,19 +336,19 @@ class optionDialog (mDialog):
     def set_value (self, value):
         self.ret=value
 
-class progressDialog (mDialog):
+class ProgressDialog (ModalDialog):
 
     """A dialog to show a progress bar"""
     
     def __init__ (self, title="", okay=True, label="", sublabel=False, parent=None,
-                  cancel=False, stop=True, pause=True,modal=False):
+                  cancel=False, stop=False, pause=False,modal=False):
         """stop,cancel,and pause will be given as callbacks to their prospective buttons."""
         self.custom_pausecb=pause
         self.custom_cancelcb=cancel
         self.custom_pause_handlers = []
         self.custom_stop_handlers = []
         self.custom_stopcb=stop
-        mDialog.__init__(self, title, okay=okay, label=label, sublabel=sublabel, parent=parent,
+        ModalDialog.__init__(self, title, okay=okay, label=label, sublabel=sublabel, parent=parent,
                          cancel=cancel,modal=modal)
         self.set_title(label)
         self.progress_bar = gtk.ProgressBar()
@@ -345,12 +357,13 @@ class progressDialog (mDialog):
         self.vbox.add(self.detail_label)
         self.detail_label.set_use_markup(True)
         self.detail_label.set_padding(H_PADDING,Y_PADDING)
-        self.detail_label.set_line_wrap(True)
+        self.detail_label.set_line_wrap(gtk.WRAP_WORD)
         self.vbox.show_all()
         if okay: self.set_response_sensitive(gtk.RESPONSE_OK,False) # we're false by default!
+        if not stop: self.stop.hide()
+        if not pause: self.pause.hide()
 
     def reset_label (self, label):
-        if not is_markup(label): label=xml.sax.saxutils.escape(label)
         self.set_title(label)
         self.label.set_text('<span weight="bold" size="larger">%s</span>'%label)
         self.label.set_use_markup(True)
@@ -386,9 +399,11 @@ class progressDialog (mDialog):
             # we keep a list of handlers for possible disconnection later
             self.custom_pause_handlers.append(self.pause.connect('toggled',self.custom_pausecb))
             self.pause.set_property('visible',True)
-        else: self.pause.set_property('visible',False)
+        else:
+            self.pause.set_property('visible',False)
+            self.pause.hide()
         # setup stop button
-        self.stop = gtk.Button(_('_Stop'))        
+        self.stop = gtk.Button(_('_Stop'))
         self.action_area.pack_end(self.stop)
         if self.custom_stopcb:
             self.stop.set_property('visible',True)
@@ -397,12 +412,22 @@ class progressDialog (mDialog):
             #self.custom_stop_handlers.append(self.stop.connect('clicked',self.cancelcb))
         else:
             self.stop.set_property('visible',False)
-        mDialog.setup_buttons(self,cancel,okay)
+            self.stop.hide()
+        ModalDialog.setup_buttons(self,cancel,okay)
         if self.custom_cancelcb:
             self.cancelcb = self.custom_cancelcb
-            #self.cancel.connect('clicked',self.custom_cancelcb)
-        
-class preferences_dialog (mDialog):
+            #self.cancel.connect('clicked',self.custom_cancelcb)            
+
+    def set_progress (self, prog, message=None):
+        if prog < 0:
+            self.progress_bar.pulse()
+        else:
+            self.progress_bar.set_fraction(prog)
+        if message: self.progress_bar.set_text(message)
+        if prog==1:
+            self.set_response_sensitive(gtk.RESPONSE_OK,True)
+
+class PreferencesDialog (ModalDialog):
 
     """A dialog to get preferences from a user and return user preferences as a list."""
     
@@ -410,7 +435,7 @@ class preferences_dialog (mDialog):
                   value_label="Value", default=True, label=None,
                   apply_func=None, parent=None, dont_ask_cb=None,
                   dont_ask_custom_text=None):
-        """Options is a list of lists where each list is ['option', VALUE], handed to OptionTable
+        """Options is a tuple of lists where each list is ['option', VALUE], handed to OptionTable
         
         VALUE can be any of the following:
         a string (will be editable)
@@ -424,7 +449,7 @@ class preferences_dialog (mDialog):
         else: modal=True
         self.apply_func = apply_func
         self.options = options        
-        mDialog.__init__(self, okay=True, label=label, parent=parent, modal=modal)
+        ModalDialog.__init__(self, okay=True, label=label, parent=parent, modal=modal)
         self.table = optionTable.OptionTable(options=self.options,
                                              option_label=option_label,
                                              value_label=value_label,
@@ -452,7 +477,7 @@ class preferences_dialog (mDialog):
         else:
             self.changedcb=None
             self.set_modal(False)
-        mDialog.setup_buttons(self, cancel, okay)
+        ModalDialog.setup_buttons(self, cancel, okay)
 
     def revertcb (self, *args):
         self.table.revert()
@@ -482,24 +507,19 @@ class preferences_dialog (mDialog):
             self.ret = self.table.options
             self.hide()
             gtk.main_quit()
-
+                   
     def cancelcb (self, *args):
         self.hide()
         self.ret=None
-        if not self.apply_func:
-            self.hide()
-            gtk.main_quit()
-        else:
-            self.hide()
         
-class booleanDialog (messageDialog):
+class BooleanDialog (MessageDialog):
     def __init__ (self, title="", default=True, label=_("Do you really want to do this"),
                   sublabel=False, cancel=True,
                   parent=None, custom_yes=None, custom_no=None, expander=None,
                   dont_ask_cb=None, dont_ask_custom_text=None,
                   cancel_returns=None, message_type=gtk.MESSAGE_QUESTION
                   ):
-        """Setup a booleanDialog which returns True or False.
+        """Setup a BooleanDialog which returns True or False.
         parent is our parent window.
         custom_yes is custom text for the button that returns true or a dictionary
                    to be handed to gtk.Button as keyword args.
@@ -515,16 +535,16 @@ class booleanDialog (messageDialog):
         self.yes,self.no = custom_yes,custom_no
         if not self.yes: self.yes = gtk.STOCK_YES
         if not self.no: self.no = gtk.STOCK_NO
-        messageDialog.__init__(self,title=title,okay=False,label=label, cancel=cancel, sublabel=sublabel,parent=parent, expander=expander, message_type=message_type)
+        MessageDialog.__init__(self,title=title,okay=False,label=label, cancel=cancel, sublabel=sublabel,parent=parent, expander=expander, message_type=message_type)
         self.responses[gtk.RESPONSE_YES]=self.yescb
         self.responses[gtk.RESPONSE_NO]=self.nocb
-        # if there's no cancel, all cancel-like actions
-        # are the equivalent of a NO response
-        self.responses[gtk.RESPONSE_NONE]= cancel and self.cancelcb or self.nocb
-        self.responses[gtk.RESPONSE_CANCEL]= cancel and self.cancelcb or self.nocb
-        self.responses[gtk.RESPONSE_CLOSE]= cancel and self.cancelcb or self.nocb
-        self.responses[gtk.RESPONSE_DELETE_EVENT]=cancel and self.cancelcb or self.nocb
-        # Setup a "Don't ask me again" button
+        if not cancel:
+            # if there's no cancel, all cancel-like actions
+            # are the equivalent of a NO response
+            self.responses[gtk.RESPONSE_NONE]=self.nocb
+            self.responses[gtk.RESPONSE_CANCEL]=self.nocb
+            self.responses[gtk.RESPONSE_CLOSE]=self.nocb
+            self.responses[gtk.RESPONSE_DELETE_EVENT]=self.nocb
         if dont_ask_cb:
             if not dont_ask_custom_text:
                 dont_ask_custom_text=_("Don't ask me this again.")
@@ -539,7 +559,7 @@ class booleanDialog (messageDialog):
         #self.action_area.show_all()
 
     def setup_buttons (self,*args,**kwargs):
-        messageDialog.setup_buttons(self,*args,**kwargs)
+        MessageDialog.setup_buttons(self,*args,**kwargs)
         self.add_button((self.no or gtk.STOCK_NO),gtk.RESPONSE_NO)
         self.add_button((self.yes or gtk.STOCK_YES),gtk.RESPONSE_YES)
 
@@ -556,7 +576,7 @@ class booleanDialog (messageDialog):
         self.ret=False
         self.okcb()
 
-class SimpleFaqDialog (mDialog):
+class SimpleFaqDialog (ModalDialog):
     """A dialog to view a plain old text FAQ in an attractive way"""
 
     INDEX_MATCHER = re.compile("^[0-9]+[.][A-Za-z0-9.]* .*")
@@ -571,8 +591,8 @@ class SimpleFaqDialog (mDialog):
                   jump_to = None,
                   parent=None,
                   modal=True):
-        print faq_file
-        mDialog.__init__(self,title=title,parent=parent,modal=modal,cancel=False)
+        #print faq_file
+        ModalDialog.__init__(self,title=title,parent=parent,modal=modal,cancel=False)
         self.set_default_size(950,500)        
         self.textview = gtk.TextView()
         self.textview.set_editable(False)
@@ -691,9 +711,79 @@ class SimpleFaqDialog (mDialog):
         if not itr:
             itr = self.textbuf.get_iter_at_offset(0)
         match_start,match_end=itr.forward_search(txt,gtk.TEXT_SEARCH_VISIBLE_ONLY)
-        print 'match_start = ',match_start
+        #print 'match_start = ',match_start
         self.textview.scroll_to_iter(match_start,False,use_align=True,yalign=0.1)
         
+class RatingsConversionDialog (ModalDialog):
+    """A dialog to allow the user to select the number of stars
+    distinct ratings should convert to.
+
+    This dialog exists to aid conversion of ratings from old gourmet
+    exports or databases or from other imports that use strings of
+    some kind ('great','groovy',etc.) to aid in conversion.
+    """
+    def __init__ (self,
+                  strings,
+                  star_generator,
+                  defaults={_("excellent"):10,
+                            _("great"):8,
+                            _("good"):6,
+                            _("fair"):4,
+                            _("poor"):2,},
+                  parent=None,
+                  modal=True):
+        """strings is a list of strings that are currently used for ratings.
+
+        The user will be asked to give the star equivalent of each string.
+        """
+        self.strings = strings
+        self.star_generator=star_generator
+        self.defaults = defaults
+        ModalDialog.__init__(
+            self,
+            title=_("Convert ratings to 5 star scale."),
+            label=_("Convert ratings."),
+            sublabel=_("Please give each of the ratings an equivalent on a scale of 1 to 5"),
+            parent=parent,
+            modal=modal
+            )
+        self.set_default_size(750,500)
+        self.ret = {}
+        self.setup_tree()
+
+    def setup_tree (self):
+        self.tv = gtk.TreeView()
+        self.setup_model()
+        self.tv.set_model(self.tm)
+        from ratingWidget import TreeWithStarMaker
+        textcol = gtk.TreeViewColumn(_('Current Rating'),gtk.CellRendererText(),text=0)
+        textcol.set_sort_column_id(0)
+        self.tv.append_column(textcol)
+        TreeWithStarMaker(self.tv,
+                          self.star_generator,
+                          col_title=_("Rating out of 5 Stars"),
+                          col_position=-1,
+                          data_col=1,
+                          handlers=[self.ratings_change_cb],
+                          )
+        self.sw = gtk.ScrolledWindow()
+        self.sw.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
+        self.sw.add(self.tv)
+        self.vbox.add(self.sw)
+        self.sw.show_all()
+
+    def setup_model (self):
+        self.tm = gtk.ListStore(str,int)
+        for s in self.strings:
+            val=self.defaults.get(s.lower(),0)
+            self.tm.append([s,val])
+            self.ret[s]=val
+
+    def ratings_change_cb (self, value, model, treeiter, colnum):
+        string = model.get_value(treeiter,0)
+        self.ret[string]=value
+        model.set_value(treeiter,colnum,value)
+
 def show_traceback (label="Error", sublabel=None):
     """Show an error dialog with a traceback viewable."""
     from StringIO import StringIO
@@ -709,14 +799,14 @@ def show_traceback (label="Error", sublabel=None):
 
 def show_message (*args, **kwargs):
     """Show a message dialog.
-    Args and Kwargs are handed to messageDialog
+    Args and Kwargs are handed to MessageDialog
     We most likely want to hand it label= and sublabel=
     """
     #if not kwargs.has_key(message_type):
     #    message_type=gtk.MESSAGE_INFO
     if not kwargs.has_key('cancel'):
         kwargs['cancel']=False
-    d=messageDialog(*args,**kwargs)
+    d=MessageDialog(*args,**kwargs)
     d.run()
     return d
 
@@ -732,7 +822,7 @@ def select_file (title,
                  select_multiple=False,
                  buttons=None
                  ):
-    sfd=select_file_dialog(title,filename=filename,filters=filters,select_multiple=select_multiple,
+    sfd=FileSelectorDialog(title,filename=filename,filters=filters,select_multiple=select_multiple,
                            action=action,set_filter=set_filter,buttons=buttons)
     return sfd.run()
 
@@ -746,7 +836,7 @@ def saveas_file (title,
                  show_filetype=True):
     """Almost identical to select_file, except that we return a tuple containing
     the filename and the export type (the string the user selected)"""
-    sfd=select_file_dialog(title,filename=filename,filters=filters,
+    sfd=FileSelectorDialog(title,filename=filename,filters=filters,
                            action=action,set_filter=set_filter,buttons=buttons,
                            show_filetype=show_filetype,parent=parent)
     retval = sfd.run()
@@ -765,10 +855,10 @@ def select_image (title,
                   filename=None,
                   action=gtk.FILE_CHOOSER_ACTION_OPEN,
                   buttons=None):    
-    sfd=select_image_dialog(title,filename=filename,action=action,buttons=buttons)
+    sfd=ImageSelectorDialog(title,filename=filename,action=action,buttons=buttons)
     return sfd.run()
 
-class select_file_dialog:
+class FileSelectorDialog:
     """A dialog to ask the user for a file. We provide a few custom additions to the
     standard file dialog, including a special choose-filetype menu and including dynamic update
     of the filetype based on user input of an extension"""
@@ -1009,7 +1099,7 @@ class select_file_dialog:
         self.fsd.destroy()
         
     
-class select_image_dialog (select_file_dialog):
+class ImageSelectorDialog (FileSelectorDialog):
     IMAGE_FILTERS = [
         ['Image',
          ['image/jpeg','image/png','image/tiff',
@@ -1032,7 +1122,7 @@ class select_image_dialog (select_file_dialog):
                   set_filter=True,
                   buttons=None
                   ):
-        select_file_dialog.__init__(self, title, filename, filters, action, set_filter, buttons)
+        FileSelectorDialog.__init__(self, title, filename, filters, action, set_filter, buttons)
 
     def post_dialog (self):
         self.preview = gtk.Image()
@@ -1051,40 +1141,62 @@ class select_image_dialog (select_file_dialog):
             self.preview.hide()
 
 def getNumber (*args, **kwargs):
-    """Run numberDialog, passing along all args, waiting on input and passing along
+    """Run NumberDialog, passing along all args, waiting on input and passing along
     the results."""
-    d = numberDialog(*args, **kwargs)
+    d = NumberDialog(*args, **kwargs)
     return d.run()
                      
 def getEntry (*args, **kwargs):
-    """Run entryDialog, passing along all args, waiting on input and passing along
+    """Run EntryDialog, passing along all args, waiting on input and passing along
     the results."""    
-    d = entryDialog(*args, **kwargs)
+    d = EntryDialog(*args, **kwargs)
     return d.run()
 
 def getBoolean (*args,**kwargs):
-    """Run booleanDialog, passing along all args, waiting on input and
+    """Run BooleanDialog, passing along all args, waiting on input and
     passing along the results."""
-    d = booleanDialog(*args,**kwargs)
+    d = BooleanDialog(*args,**kwargs)
     retval = d.run()
     if retval==None:
-        raise "getBoolean dialog cancelled!"
+        raise UserCancelledError("getBoolean dialog cancelled!")
     else:
         return retval
 
 def getOption (*args,**kwargs):
-    d=optionDialog(*args,**kwargs)
+    d=OptionDialog(*args,**kwargs)
     return d.run()
 
 def getRadio (*args,**kwargs):
-    d=radioDialog(*args,**kwargs)
+    d=RadioDialog(*args,**kwargs)
     return d.run()
 
 def show_faq (*args,**kwargs):
     d=SimpleFaqDialog(*args,**kwargs)
     return d.run()
 
+def get_ratings_conversion (*args,**kwargs):
+    d=RatingsConversionDialog(*args,**kwargs)
+    return d.run()
+
+def show_amount_error (txt):
+    """Show an error that explains how numeric amounts work."""
+    de.show_message(label=_("""I'm sorry, I can't understand
+the amount "%s".""")%txt,
+                    sublabel=_("Amounts must be numbers (fractions or decimals), ranges of numbers, or blank."),
+                    expander=[_("_Details"),
+                              _("""
+The "unit" must be in the "unit" field by itself.
+For example, if you want to enter one and a half cups,
+the amount field could contain "1.5" or "1 1/2". "cups"
+should go in the separate "unit" field.
+
+To enter a range of numbers, use a "-" to separate them.
+For example, you could enter 2-4 or 1 1/2 - 3 1/2.
+""")])
+
+
 if __name__ == '__main__':
+    import testExtras
     w=gtk.Window()
     w.connect('delete_event',gtk.main_quit)
     b=gtk.Button("show dialog (modal)")
@@ -1092,7 +1204,7 @@ if __name__ == '__main__':
           ["String Option","Hello"],
           ["Integer Option",1],
           ["Float Option",float(3)])
-    pd=preferences_dialog(options=opts)
+    pd=PreferencesDialog(options=opts)
     def run_prefs (*args):
          pd.run()
     b.connect('clicked',run_prefs)
@@ -1106,13 +1218,17 @@ if __name__ == '__main__':
     char_measure = ""
     for n in range(10): char_measure="%s %s"%(char_measure,n)
     char_measure = char_measure * 50
+    from ratingWidget import StarGenerator
     for s,f in [
         ['show dialog (modal)',run_prefs],
-        ['show dialog (not modal)',lambda *args: preferences_dialog(options=opts,apply_func=show_options).show()],
+        ['show ratings dialog',lambda *args: get_ratings_conversion(['Good','Great','So so','Hot shit'],
+                                                                    StarGenerator(),
+                                                                    )],
+        ['show dialog (not modal)',lambda *args: PreferencesDialog(options=opts,apply_func=show_options).show()],
         ['show FAQ',lambda *args: show_faq(jump_to='shopping')],
-        ['show message',lambda *args: show_message('howdy',label='Hello there. This is a very long label for the top of a dialog & it has invalid XML.', sublabel='And this is a sub message & it has < lots > of invalid >>> xml.',message_type=gtk.MESSAGE_WARNING)],
-        ['show marked up message',lambda *args: show_message('howdy',label='Hello <i>there</i>. This is a very long label for the top of a dialog &amp; it has valid XML.', sublabel='And this <u>is</u> a sub message.',message_type=gtk.MESSAGE_WARNING)],
+        ['show message',lambda *args: show_message('howdy',label='Hello there. This is a very long label for the top of a dialog.', sublabel='And this is a sub message.',message_type=gtk.MESSAGE_WARNING)],
         ['get entry', lambda *args: getEntry(label='Main label',sublabel='sublabel',entryLabel='Entry Label: ')],
+        ['get number', lambda *args: getNumber(label='Main label',sublabel='sublabel')],        
         ['get long entry', lambda *args: getEntry(label='Main label', sublabel=char_measure, entryLabel='Entry Label: ',default_character_width=75,entryTip='Enter something long here.')],
         ['show boolean', lambda *args: getBoolean()],
         ['show custom boolean', lambda *args: getBoolean(custom_yes='_Replace',
@@ -1120,10 +1236,10 @@ if __name__ == '__main__':
                                                          cancel=False
                                                          )],
         ['show radio dialog', lambda *args: getRadio(label='Main label',
-                                                     sublabel='sublabel'*10,
+                                                     sublabel='sublabel'*10,default=2,
                                                      options=[('First',1),
                                                               ('Second',2),
-                                                              ('Third',3)])],
+                                                              ('Third',3)]),],
         ['get image dialog',lambda *args: msg(select_image('Select Image'))],
         ['get file dialog',lambda *args: msg(select_file('Select File',
                                                      filters=[['Plain Text',['text/plain'],['*.txt','*.TXT']],
@@ -1142,8 +1258,15 @@ if __name__ == '__main__':
                                             ['Web Page (HTML)',['text/html'],['*.htm','*.HTM','*.html','*.HTML']],
                                             ['Mealmaster File',['text/mmf'],['*.mmf','*.MMF']]]))],
         ]:
+        print b,f,s
         b = gtk.Button(s)
-        b.connect('clicked',f)
+        def wrap (f):
+            def _ (*args,**kwargs):
+                print 'Doing ',f
+                print 'f returns:',f()
+                print 'Done'
+            return _
+        b.connect('clicked',wrap(f))
         vb.add(b)
     w.add(vb)
     vb.show_all()
