@@ -42,6 +42,16 @@ class importer:
         # allow threaded calls to be terminated (this
         # has to be implemented in subclasses).
         self.terminated = False
+        # Our rating converter -- if we've been handed a class, we
+        # assume our caller will handle doing the
+        # conversion. Otherwise we do it ourselves.
+        if rating_converter:
+            self.rating_converter = rating_converter
+            self.do_conversion = False
+        else:
+            self.rating_converter = RatingConverter()
+            self.do_conversion = True
+        
         if hasattr(self.rd,'km'):
             debug('Using existing keymanager',2)
             self.km=self.rd.km
@@ -70,7 +80,10 @@ class importer:
         #print_timer_info()
 
     def _run_cleanup_ (self):
-        pass
+        if self.do_conversion:
+            # if we have something to convert
+            if self.rating_converter.to_convert:
+                self.rating_converter.do_conversions(self.rd)
 
     def check_for_sleep (self):
         timeaction = TimeAction('importer.check_for_sleep',10)
@@ -99,7 +112,7 @@ class importer:
         else:
             self.rec = {}
         #if not self.rec.has_key('id'):
-        # always create a new ID        
+        # always create a new ID     
         if self.rec.has_key('id'):
             if self.id_converter.has_key(self.rec['id']):
                 self.rec['id']=self.id_converter[self.rec['id']]
@@ -126,6 +139,9 @@ class importer:
 
     def commit_rec (self):
         timeaction = TimeAction('importer.commit_rec',10)
+        for key in ['cuisine','category','title']:
+            if self.rec.has_key(key):
+                self.rec[key]=re.sub('\s+',' ',self.rec[key]).strip()
         # if servings can't be recognized as a number, add them to the
         # instructions.
         if self.rec.has_key('servings'):
@@ -134,15 +150,41 @@ class importer:
                 self.rec['servings']=str(servs)
             else:
                 self._move_to_instructions(self.rec,'servings')
+        # Check preptime and cooktime
+        for t in ['preptime','cooktime']:
+            if self.rec.has_key(t) and type(self.rec[t])!=int:
+                secs = self.conv.timestring_to_seconds(self.rec[t])
+                if secs:
+                    self.rec[t]=secs
+                else:
+                    self._move_to_instructions(self.rec,t)
         # Markup instructions and mods as necessary
         if self.do_markup:
             for k in ['instructions','modifications']:
                 if self.rec.has_key(k): self.rec[k] = xml.sax.saxutils.escape(self.rec[k])
+        # A little strange, this is, but for UI reasons, we want to
+        # keep track of any ratings that are not integers so that we
+        # can ask the user how to convert them when we're all done
+        # with importing.              
+        remembered_rating = None
+        if self.rec.has_key('rating') and type(self.rec['rating']) not in [int,float]:
+            if string_to_rating(self.rec['rating']):
+                self.rec['rating']=string_to_rating(self.rec['rating'])
+            else:
+                remembered_rating = self.rec['rating']
+                del self.rec['rating']
         tt=TimeAction('importer.commit_rec - rd.add_rec',5)
         debug('commiting recipe %s'%self.rec,0)
+        # Check for images without thumbnails
+        if self.rec.get('image',None) and not self.rec.get('thumb',None):
+            print 'Adding thumbnail.'
+            img = ImageExtras.get_image_from_string(self.rec['image'])
+            thumb = ImageExtras.resize_image(img,40,40)
+            self.rec['thumb'] = ImageExtras.get_string_from_image(thumb)
         r = self.rd.add_rec(self.rec)
         tt.end()
         self.added_recs.append(r)
+        if remembered_rating: self.rating_converter.add(r.id,remembered_rating)
         self.check_for_sleep()
         timeaction.end()
         self.rec_timer.end()
@@ -180,6 +222,10 @@ class importer:
                  
     def commit_ing (self):
         timeaction = TimeAction('importer.commit_ing 1',10)
+        # Strip whitespace...
+        for key in ['item','ingkey','unit']:
+            if self.ing.has_key(key):
+                self.ing[key]=re.sub('\s+',' ',self.ing[key]).strip()
         if not (
             (self.ing.has_key('refid') and
              self.ing['refid'])
@@ -189,7 +235,7 @@ class importer:
             ):
             #self.ing['ingkey']=self.km.get_key(self.ing['item'],0.9)
             if self.ing.has_key('item'):
-                self.ing['ingkey']=self.km.get_key(self.ing['item'])
+                self.ing['ingkey']=self.km.get_key_fast(self.ing['item'])
             else:
                 debug('Ingredient has no item! %s'%self.ing,-1)
         timeaction.end()
@@ -277,6 +323,13 @@ simple_matcher = re.compile(
     '(%(NUMBER_REGEXP)s+)\s*/\s*([\d]+)'%locals()
     )
 
+def string_to_rating (s):
+    m = simple_matcher.match(s)
+    if m:
+        top=float(convert.frac_to_float(m.groups()[0]))
+        bottom = float(convert.frac_to_float(m.groups()[2]))
+        return int(top/bottom * 10)
+
             
         
         
@@ -317,12 +370,12 @@ class MultipleImporter:
             if self.total_size: self.current_percentage = float(self.sizes.get(fn,-1))/self.total_size
             else: self.current_percentage = -1
             gt.gtk_enter()
-            if self.grm.prog_dialog.detail_label:
-                self.grm.prog_dialog.detail_label.set_text(_('<i>Importing %s</i>')%fn)
-                self.grm.prog_dialog.detail_label.set_use_markup(True)
-            #self.grm.prog_dialog.label.set_text(_('<i>Importing %s</i>')%fn)
+            if self.grm.progress_dialog.detail_label:
+                self.grm.progress_dialog.detail_label.set_text(_('<i>Importing %s</i>')%fn)
+                self.grm.progress_dialog.detail_label.set_use_markup(True)
+            #self.grm.progress_dialog.label.set_text(_('<i>Importing %s</i>')%fn)
             gt.gtk_leave()
-            #print 'Using import class ',ic
+            print 'Using import class ',ic
             kwargs['progress']=self.show_progress
             self.iclass = ic(*args,**kwargs)
             self.suspend = self.iclass.suspend
@@ -404,5 +457,113 @@ class Tester:
         else:
             self.ofi.seek(0)
             
+class RatingConverter:
+
+    """A class to handle converting ratings from strings to integers.
+
+    This is here since many of our imports will grab ratings for us in
+    the form of 'excellent' or some such, and we'll want to let our
+    user convert those to stars as they see fit.
+    """
+    
+    conversions = {'excellent':5,
+                   'great':4,
+                   'good':3,
+                   'fair':2,
+                   'okay':2,
+                   'poor':1,
+                   _('Excellent').lower():5,
+                   _('Great').lower():4,
+                   _('Good').lower():3,
+                   _('Fair').lower():2,
+                   _('Poor').lower():1,
+                   _('Okay').lower():2,
+                   }
+    
+
+    def __init__ (self):
+        self.to_convert = {}
+        self.got_conversions = False
+
+    def add (self, id, rating):
+        if type(rating)==int:
+            raise "Why do you need me? id: %(id)s rating: %(rating)s"%locals()
+        self.to_convert[id]=rating
+
+    def get_conversions (self, star_generator=None):
+        """Get our conversions.
+
+        If necessary, ask user to convert for us.
+        """
+        self.got_conversions=True
+        if not star_generator:
+            from gourmet.ratingWidget import star_generator
+        ratings = []
+        need_conversions = False
+        for v in self.to_convert.values():
+            if not need_conversions and not self.conversions.has_key(v.lower()):
+                need_conversions = True
+            if not v in ratings: ratings.append(v)
+        if need_conversions:
+            self.conversions = de.get_ratings_conversion(ratings,star_generator,defaults=self.conversions)
+
+    def do_conversions (self, db):
+        if not self.got_conversions:
+            print 'Get conversion!'
+            self.get_conversions()
+        for id,rating in self.to_convert.items():
+            try:
+                db.modify_rec(db.get_rec(id),{'rating':self.conversions[str(rating).lower()]})
+            except:
+                print 'wtf... problem with rating ',rating,'for recipe',id
+                raise
+
+import unittest
+
+class RatingConverterTest (unittest.TestCase):
+
+    def setUp (self):
+
+        class FakeDB:
+
+            recs = dict([(n,{}) for n in range(20)])
+            
+            def get_rec (self, n): return n
+            def modify_rec (self, n, d):
+                for attr,val in d.items(): self.recs[n][attr]=val
+
+        self.db = FakeDB()
+
+    def testAutomaticConverter (self):
+        rc = RatingConverter()
+        tests = ['good','Great','Excellent','poor','okay']
+        for n,rating in enumerate(tests):
+            rc.add(n,rating)
+            self.db.recs[n]['rating']=rating
+        total = n
+        rc.do_conversions(self.db)
+        print 'Conversions: '
+        for n,rating in enumerate(tests):
+            print 'Converted',rating,'->',self.db.recs[n]['rating']
+
+    def testInteractiveConverter (self):
+        rc = RatingConverter()
+        tests = ['alright','pretty good','funny tasting',
+                 'okeydokey','not bad','damn good.']
+        for n,rating in enumerate(tests):
+            rc.add(n,rating)
+            self.db.recs[n]['rating']=rating
+        total = n
+        rc.do_conversions(self.db)
+        #print 'Conversions: '
+        #for n,rating in enumerate(tests):
+        #    print 'Converted',rating,'->',self.db.recs[n]['rating']
+
+    def testStringToRatingConverter (self):
+        assert(string_to_rating('4/5 stars')==8)
+        assert(string_to_rating('3 1/2 / 5 stars')==7)
+        assert(string_to_rating('4/10 stars')==4)
+
+
 if __name__ == '__main__':
     unittest.main()
