@@ -7,6 +7,8 @@ import exporters.printer as printer
 from gdebug import *
 from gglobals import *
 from gettext import gettext as _
+from nutrition.nutritionLabel import NutritionLabel
+from nutrition.nutrition import NutritionInfoList
 from FauxActionGroups import ActionManager
 import mnemonic_manager
 
@@ -21,6 +23,7 @@ class ShopGui (ActionManager):
         self.mm.fix_conflicts_peacefully()
         self.conv=conv
         self.rg = RecGui
+        self.nd = self.rg.nd # convenient reference to our nutrition
         self.recs = {}
         self.extras = []
         self.data,self.pantry=self.grabIngsFromRecs([])
@@ -56,7 +59,8 @@ class ShopGui (ActionManager):
             'shopClear' : self.clear,
             'move_to_shopping' : self.rem_selection_from_pantry,
             'move_to_pantry' : self.add_selection_to_pantry,
-            'show_help': lambda *args: de.show_faq(HELP_FILE,jump_to='Shopping')
+            'show_help': lambda *args: de.show_faq(HELP_FILE,jump_to='Shopping'),
+            'show_nutritional_info': self.show_nutritional_info,
             })
 
     def init_action_manager (self):
@@ -118,6 +122,31 @@ class ShopGui (ActionManager):
             mod.set_value(iter,2,convert.float_to_frac(mult))
         return mod
 
+    def show_nutritional_info (self, *args):
+        """Show nutritional information for this shopping list.
+        """
+        nl = NutritionLabel(self.rg.prefs,
+                            custom_label=_('Nutritional Information for Shopping List')
+                            )
+        ings = []
+        for d in [self.sh.dic,self.sh.mypantry]:
+            for k,units in d.items():
+                for a,u in units:
+                    ings.append([k,a,u])
+        nutinfo = NutritionInfoList(
+            [self.nd.get_nutinfo_for_item(*i) for i in ings]
+            )
+        nl.set_nutinfo(nutinfo)
+        sw = gtk.ScrolledWindow()
+        sw.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
+        sw.add_with_viewport(nl)
+        sw.show(); nl.show()
+        md = de.ModalDialog(title=_("Nutritional Information for Shopping List"),modal=False)
+        md.vbox.pack_start(sw,fill=True,expand=True)
+        md.set_default_size(600,500)
+        nl.show()
+        md.run()
+
     def save (self, *args):
         debug("save (self, *args):",5)
         self.message('Saving')
@@ -139,12 +168,12 @@ class ShopGui (ActionManager):
         #of.close()
         import exporters.lprprinter
         self._printList(exporters.lprprinter.SimpleWriter,file=filename,show_dialog=False)
+
     def printList (self, *args):
         debug("printList (self, *args):",0)
         self._printList(printer.SimpleWriter,dialog_parent=self.widget)
         
     def _printList (self, printer, *args, **kwargs):
-        #w=printer.SimpleWriter(*args,**kwargs)
         w = printer(*args,**kwargs)
         w.write_header(_("Shopping list for %s")%time.strftime("%x"))
         w.write_subheader(_("For the following recipes:"))
@@ -157,7 +186,6 @@ class ShopGui (ActionManager):
         self.sh.list_writer(w.write_subheader,write_itm)
         w.close()
 
-        
     def getSelectedRecs (self):
         """Return each recipe in list"""
         def foreach(model,path,iter,recs):
@@ -235,12 +263,11 @@ class ShopGui (ActionManager):
         self.slMod = self.createIngModel(data)
         self.slTree = self.create_ingTree(self.glade.get_widget('shopITree'),
                                      self.slMod)
-        self.slTree.connect('row-activated',self.popup_ing_menu)
         self.slTree.connect('popup-menu',self.popup_ing_menu)
         def slTree_popup_cb (tv, event):
             debug("slTree_popup_cb (tv, event):",5)
-            if event.button==3:
-                self.popup_ing_menu(tv)
+            if event.button==3 or event.type == gtk.gdk._2BUTTON_PRESS:
+                self.popup_ing_menu(tv,event)
                 return True
         self.slTree.connect('button-press-event',slTree_popup_cb)
         self.slTree.get_selection().connect('changed',self.slTree_sel_changed_cb)
@@ -252,16 +279,16 @@ class ShopGui (ActionManager):
         self.pMod = self.createIngModel(data)
         self.pTree = self.create_ingTree(self.glade.get_widget('pantryTree'),
                                     self.pMod)
-        self.pTree.connect('row-activated',self.popup_pan_menu)
         self.pTree.connect('popup-menu',self.popup_pan_menu)
         self.pTree.get_selection().connect('changed',self.pTree_sel_changed_cb)
         # reset the first time...
         self.pTree_sel_changed_cb(self.pTree.get_selection())
         def pTree_popup_cb (tv, event):
             debug("pTree_popup_cb (tv, event):",5)
-            if event.button==3:
-                self.popup_pan_menu(tv)
+            if event.button==3 or event.type == gtk.gdk._2BUTTON_PRESS:
+                self.popup_pan_menu(tv,event)
                 return True
+            
         self.pTree.connect('button-press-event',pTree_popup_cb)
         
     def create_ingTree (self, widget, model):
@@ -360,15 +387,31 @@ class ShopGui (ActionManager):
         debug("ingSelection returns: %s"%selected_keys,3)
         return selected_keys
 
-    def popup_ing_menu (self, tv, *args):
+    def popup_ing_menu (self, tv, event=None, *args):
         debug("popup_ing_menu (self, tv, *args):",5)
         self.tv = tv
-        self.pop.popup(None,None,None,0,0)
+        if not event:
+            event = gtk.get_current_event()
+        t = (event and hasattr(event,'time') and getattr(event,'time')
+                or 0)
+        btn = (event and hasattr(event,'button') and getattr(event,'button')
+               or 0)
+        print 'self.pop.popup(None,None,None,',btn,t,')'        
+        self.pop.popup(None,None,None,btn,t)
+        return True
 
-    def popup_pan_menu (self, tv, *args):
+    def popup_pan_menu (self, tv, event=None, *args):
         debug("popup_pan_menu (self, tv, *args):",5)
         self.tv = tv
-        self.panpop.popup(None,None,None,0,0)
+        if not event:
+            event = gtk.get_current_event()
+        t = (event and hasattr(event,'time') and getattr(event,'time')
+                or 0)
+        btn = (event and hasattr(event,'button') and getattr(event,'button')
+               or 0)
+        print 'self.panpop.popup(None,None,None,',btn,t,')'
+        self.panpop.popup(None,None,None,btn,t)
+        return True
 
     def setup_popup (self):
         debug("setup_popup (self):",5)
@@ -635,12 +678,12 @@ class ShopGui (ActionManager):
         self.stat.push(self.contid,txt)
         self.rg.message(txt)
     
-class OptionalIngDialog (de.mDialog):
+class OptionalIngDialog (de.ModalDialog):
     """A dialog to query the user about whether to use optional ingredients."""
     def __init__ (self,vw,prefs,rg,mult=1,default=False):
         debug("__init__ (self,vw,default=False):",5)
         self.rg=rg
-        de.mDialog.__init__(
+        de.ModalDialog.__init__(
             self, default,
             label=_("Select optional ingredients."),
             sublabel=_("Please specify which of the following optional ingredients you'd like to include on your shopping list."))
@@ -754,7 +797,13 @@ class shopIngredientEditor (reccard.IngredientEditor):
         self.curkey = self.keyBox.entry.get_text()
         if self.curkey == self.last_key:
             return
-        self.set_keymodel(ing=self.getKeyList(self.curkey))
+        def vis (m, iter):
+            x= m.get_value(iter,0)
+            if x.find(self.curkey) > -1:
+                return True
+            else: return False
+        self.keyBox.get_model().set_visible_func(vis)
+        self.keyBox.get_model().refilter()
         self.last_key=self.curkey
         
     def add (self, *args):
@@ -800,14 +849,17 @@ def getOptionalIngDic (ivw, mult, prefs, rg):
     We look at prefs to see if 'shop_always_add_optional' is set, in which case
     we don't ask our user."""    
     debug("getOptionalIngDic (ivw):",5)
-    vw = ivw.select(optional=True)
+    #vw = ivw.select(optional=True)
+    vw = filter(lambda r: r.optional==True, ivw)
     # optional_mode: 0==ask, 1==add, -1==dont add
     optional_mode=prefs.get('shop_handle_optional',0)
     if optional_mode:
-        if optional_mode==1: return True
-        elif optional_mode==-1: return False
+        if optional_mode==1:
+            return True
+        elif optional_mode==-1:
+            return False
     elif len(vw) > 0:
-        if not 0 in [i.shopoptional for i in vw]:
+        if not None in [i.shopoptional for i in vw]:
             # in this case, we have a simple job -- load our saved
             # defaults
             dic = {}
@@ -816,9 +868,10 @@ def getOptionalIngDic (ivw, mult, prefs, rg):
                 else: dic[i.ingkey]=False
             return dic
         # otherwise, we ask our user
+        print 'Run OID with ',vw,prefs,rg,mult
         oid=OptionalIngDialog(vw, prefs, rg,mult )
         retval = oid.run()
         if retval:
-            return
+            return retval
         else:
             raise "Option Dialog cancelled!"
