@@ -1,15 +1,8 @@
 #!/usr/bin/python
-'''
-This is an INI-style section marker. It enables our NSI installer build script to pull the
-version number of the gourmet code automatically, so we don't have to edit it manually every time
-we make a new release. Do not touch. It's a comment anyway. So why would you want to mess with it?
-
-[section1]
-'''
 from gettext import gettext as _
 appname = _("Gourmet Recipe Manager")
-copyright = _("Copyright (c) 2004,2005 Thomas M. Hinkle. GNU GPL")
-version = "0.8.5.14"
+copyright = _("Copyright (c) 2004,2005,2006 Thomas M. Hinkle. GNU GPL")
+version = "0.11.2"
 # repeated in setup.cfg
 description = _("Gourmet Recipe Manager is an application to store, organize and search recipes. Gourmet also makes it easy to create shopping lists from recipes. Gourmet imports recipes from a number of sources, including MealMaster and MasterCook archives and several popular websites. Gourmet can export recipes as text, MealMaster files, HTML web pages, and a custom XML format for exchange with other Gourmet users. Gourmet supports linking images with recipes.")
 authors = ["Thomas M. Hinkle <Thomas_Hinkle@alumni.brown.edu>",
@@ -17,5 +10,6 @@ authors = ["Thomas M. Hinkle <Thomas_Hinkle@alumni.brown.edu>",
            _("Daniel Folkinshteyn <nanotube@gmail.com> (Windows installer)"),
            _("Richard Ferguson (improvements to Unit Converter interface)"),
            _("R.S. Born (improvements to Mealmaster export)"),
-           _("ixat <http://ixat.deviantart.com> (logo and splash screen)"),
+           _("ixat <ixat.deviantart.com> (logo and splash screen)"),
+           _("Yula Zubritsky (nutrition and add-to-shopping list icons)")
            ]
